/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.demo;

import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import simbad.sim.Agent;
import simbad.sim.SimpleAgent;
import simbad.sim.Arch;
import simbad.sim.Box;
import simbad.sim.RangeSensorBelt;
import simbad.sim.RobotFactory;
import simbad.sim.Wall;
import simbad.sim.StraightGoalAgent;
import simbad.sim.ChaosAgent;

/** A collision avoidance demo.
 * This demo shows several robot with sonars 
 * and bumpers performing a collision avoidance behavior*/ 
public class BusyRoomDemo extends Demo {

    public BusyRoomDemo() {
		//perimeter Walls
        Wall north = new Wall(new Vector3d(8, 0, 0), 20, 1, this);
        north.rotate90(1);
        add(north);
        Wall south = new Wall(new Vector3d(-8, 0, 0), 20, 1, this);
        south.rotate90(1);
        add(south);
        Wall east = new Wall(new Vector3d(0, 0, 10), 16, 1, this);
        add(east);
        Wall west = new Wall(new Vector3d(0, 0, -10), 16, 1, this);
        add(west);
		
		Wall w1 = new Wall(new Vector3d(-5,0,0), 6, 1, this);
		add(w1);
		Wall w2 = new Wall(new Vector3d(5,0,0), 6, 1, this);
		add(w2);


		StraightGoalAgent Robot1 = new StraightGoalAgent(new Vector3d(0,0,7), new Vector3d(0,0,-6), "straight1", 0.2);
		add(Robot1);
		
		//top 2 agents
		StraightGoalAgent Robot2 = new StraightGoalAgent(new Vector3d(-3,0,-1), new Vector3d(3,0,-3), "straight2");
		add(Robot2);
		StraightGoalAgent Robot3 = new StraightGoalAgent(new Vector3d(-6,0,-3), new Vector3d(6,0,-1), "straight3");
		add(Robot3);
		
		//bottom 2 agents
		StraightGoalAgent Robot6 = new StraightGoalAgent(new Vector3d(6,0,-1), new Vector3d(-6,0,-3), "straight6");
		add(Robot6);
		StraightGoalAgent Robot7 = new StraightGoalAgent(new Vector3d(3,0,-3), new Vector3d(-3,0,-1), "straight7");
		add(Robot7);
		
		//bottom 2 agents
		StraightGoalAgent Robot9 = new StraightGoalAgent(new Vector3d(-1,0,-6), new Vector3d(1,0,6), "straight9");
		add(Robot9);
		StraightGoalAgent Robot10 = new StraightGoalAgent(new Vector3d(1,0,-6), new Vector3d(-1,0,6), "straight10");
		add(Robot10);
    }
}