/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.demo;

import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import simbad.sim.Agent;
import simbad.sim.SimpleAgent;
import simbad.sim.Arch;
import simbad.sim.Box;
import simbad.sim.RangeSensorBelt;
import simbad.sim.RobotFactory;
import simbad.sim.Wall;
import simbad.sim.StraightGoalAgent;
import simbad.sim.ErraticGoalAgent;
import simbad.sim.ChaosAgent;

/** A collision avoidance demo.
 * This demo shows several robot with sonars 
 * and bumpers performing a collision avoidance behavior
 * Early test environment for MATLAB Link up
 * NO LONGER USED*/ 
public class NeuralAgentDemo extends Demo {

    public class Robot extends Agent {

        RangeSensorBelt sonars, bumpers;
		//MatlabNeuralInterface brain;
		int counter = 0;

        public Robot(Vector3d position, String name) {
            super(position, name);
            // Add sensors
            bumpers = RobotFactory.addBumperBeltSensor(this);
            sonars = RobotFactory.addSonarBeltSensor(this,24);
			//brain = new MatlabNeuralInterface();
        }

        /** Initialize Agent's Behavior */
        public void initBehavior() {
        // nothing particular in this case
			counter = 0;
        }

        /** Perform one step of Agent's Behavior */
        public void performBehavior() {
			//Call back to Matlab with a counter, simple example of communication to matlab
			//brain.testFunction(counter++);
            if (bumpers.oneHasHit()) {
                setTranslationalVelocity(-0.1);
                setRotationalVelocity(0.1 * Math.random());

            } else {
                // Front left obstacle ?
                if (sonars.hasHit(0) && (sonars.hasHit(1))) {
                    setRotationalVelocity(-Math.PI / 4);
                    setTranslationalVelocity(0.1);
                }
                // Front right obstacle ?
                else if (sonars.hasHit(0) && (sonars.hasHit(23))) {
                    setRotationalVelocity(Math.PI / 4);
                    setTranslationalVelocity(0.1);
                }
                // left obstacle ?
                else if (sonars.hasHit(3) || sonars.hasHit(4)) {
                    setRotationalVelocity(-Math.PI / 8);
                    setTranslationalVelocity(0.1);
                }
                // right obstacle ?
                else if (sonars.hasHit(21) || sonars.hasHit(20)) {
                    setRotationalVelocity(Math.PI / 8);
                    setTranslationalVelocity(0.1);
                } else if ((getCounter() % 100) == 0) {
                    setRotationalVelocity(Math.PI / 2 * (0.5 - Math.random()));
                    setTranslationalVelocity(0.5);
                }
            }
        }
    }

    public NeuralAgentDemo() {
        Wall north = new Wall(new Vector3d(8, 0, 0), 20, 1, this);
        north.rotate90(1);
        add(north);
        Wall south = new Wall(new Vector3d(-8, 0, 0), 20, 1, this);
        south.rotate90(1);
        add(south);
        Wall east = new Wall(new Vector3d(0, 0, 10), 16, 1, this);
        add(east);
        Wall west = new Wall(new Vector3d(0, 0, -10), 16, 1, this);
        add(west);
		//Random box and arch
        //Box b1 = new Box(new Vector3d(-3, 0, 3), new Vector3f(1, 1, 2), this);
        //add(b1);
        //Arch a1 = new Arch(new Vector3d(3, 0, -3), this);
        //add(a1);
        
		//Robot NeuralAgent1 = new Robot(new Vector3d(1, 0, 1), "avoider1");
		//add(NeuralAgent1);
		
		ChaosAgent Agent2 = new ChaosAgent(new Vector3d(2, 0, 2), "avoider1");
		add(Agent2);
		ChaosAgent Agent3 = new ChaosAgent(new Vector3d(3, 0, 3), "avoider1");
		add(Agent3);
		ChaosAgent Agent4 = new ChaosAgent(new Vector3d(4, 0, 4), "avoider1");
		add(Agent4);
		ChaosAgent Agent5 = new ChaosAgent(new Vector3d(5, 0, 5), "avoider1");
		add(Agent5);
		ChaosAgent Agent6 = new ChaosAgent(new Vector3d(6, 0, 6), "avoider1");
		add(Agent6);
		
		/*ErraticGoalAgent RobotE1 = new ErraticGoalAgent(new Vector3d(1,0,1), new Vector3d(4,0,4), "straight1", 0.1);
		add(RobotE1);
		ErraticGoalAgent RobotE2 = new ErraticGoalAgent(new Vector3d(-1,0,-1), new Vector3d(-4,0,-4), "straight2", 0.2);
		add(RobotE2);
		ErraticGoalAgent RobotE3 = new ErraticGoalAgent(new Vector3d(1,0,-1), new Vector3d(4,0,-4), "straight3", 0.3);
		add(RobotE3);
		ErraticGoalAgent RobotE4 = new ErraticGoalAgent(new Vector3d(-1,0,1), new Vector3d(-4,0,4), "straight4", 0.4);
		add(RobotE4);*/
		
		StraightGoalAgent Robot1 = new StraightGoalAgent(new Vector3d(1,0,1), new Vector3d(4,0,4), "straight1", 0.1);
		add(Robot1);
		StraightGoalAgent Robot2 = new StraightGoalAgent(new Vector3d(-1,0,-1), new Vector3d(-4,0,-4), "straight2", 0.1);
		add(Robot2);
		StraightGoalAgent Robot3 = new StraightGoalAgent(new Vector3d(1,0,-1), new Vector3d(4,0,-4), "straight3", 0.1);
		add(Robot3);
		StraightGoalAgent Robot4 = new StraightGoalAgent(new Vector3d(-1,0,1), new Vector3d(-4,0,4), "straight4", 0.1);
		add(Robot4);
    
    }
}