/* author: Louis Hugues - created on 12 fvr. 2005  */
package simbad.gui;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.Panel;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.ImageComponent2D;
import simbad.sim.EnvironmentDescription;
import simbad.sim.Simulator;
import simbad.sim.World;


/**
 * Runs simbad simulator in batch mode with no user interface (only small 3d window).
 * Using the folling scenario:
 * construct->reset->step, step ,..., step -> dispose-> System.exit
 */
public class Simbatch {
    int counter;
    Frame frame;
    World world;
    Simulator simulator;
    Canvas3D canvas3d;
    Panel panel;
    ImageComponent2D img;
    /** Construct a batch version of Simbad simulator */
    public Simbatch(EnvironmentDescription ed, boolean do3DRendering ){
        counter = 0;
        if(do3DRendering)
        	world = new World(ed);
        else
        	world = new World(ed, do3DRendering); //don't do the 3d rendering
        // !!!!
        // We need absolutly to show the 3d world in a window
        // otherwise it reveal a memory bug in java3d
        // see Bug ID: 4727054 
        // !!!!!
        canvas3d = world.getCanvas3D();
        frame = new Frame();
        panel = new Panel();
        panel.setLayout(new BorderLayout());
        panel.add(canvas3d);
        
        frame.add(panel);
        frame.pack();
        frame.setSize(200, 200);
        frame.setVisible(true); //Commenting this out causes it to not render the graphics and goes way faster.
        //canvas3d.renderOffScreenBuffer();
         if (!do3DRendering){
           //frame.hide();
           // Do not render
           //canvas3d.stopRenderer();
        	 frame.setVisible(false);
        	 panel.remove(canvas3d);
        	 img = new ImageComponent2D(ImageComponent2D.FORMAT_RGB,200,200);
        	 canvas3d.setOffScreenBuffer(img);
        	 //canvas3d.setSize(100, 100);
        	 canvas3d.getScreen3D().setSize(200, 200);
        	 canvas3d.getScreen3D().setPhysicalScreenWidth(0.1);
        	 canvas3d.getScreen3D().setPhysicalScreenHeight(0.1);
        	 canvas3d.renderOffScreenBuffer();
         }
        simulator = new Simulator(null, world, ed);
    }

    /** Restart the simulation */
    public void reset(){
        simulator.resetSimulation();
        simulator.initBehaviors();
    }
    /** perform one step - call it in your main loop*/
    public void step(){
        simulator.simulateOneStep();
        }
        
    /** Dispose resource at end.**/
    public void dispose(){
       
        simulator.dispose();
        world.dispose();
        simulator = null;
        world = null;
        canvas3d = null;
      
        frame.dispose();
        System.runFinalization();
        System.gc();
    }

}
