/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.holodeck;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import simbad.piconode.core.node.*;
import simbad.piconode.visual.NNPiconode;

public class HoloManager 
{
	final static String classNames[] = { "simbad.holodeck.Evaluator_Maze", "simbad.holodeck.Evaluator_BusyHallway", "simbad.holodeck.Evaluator_BusyHallwayReverse",
		"simbad.holodeck.Evaluator_BusyRoom"};
	
	public static String currentPath = "";
	
    /** Creates a menu corresponding to the demo set. */
    public final static JMenu createMenu(ActionListener actionListener) {
        JMenu menu = new JMenu("File");
        for (int i = 0; i < classNames.length; i++) {
            JMenuItem item = new JMenuItem(classNames[i]);
            item.setActionCommand("open");
            item.addActionListener(actionListener);
            menu.add(item);
        }
        return menu;
    }

    /** Creates a demo corresponding to the menu item. */
       public static Holo getHoloFromOpenDialogue(ActionEvent event, JFrame simulator) {
    	   
    	Class cl = null;
        Holo hologram=null;
        String holoName = ((JMenuItem) event.getSource()).getText();
        try {
        	//load the new holodeck simulation
        	cl = Class.forName(holoName);
        	hologram = (Holo)cl.newInstance();
        	
        	//since this is a load of a neural simulation a brain must be chosen.
        	JFileChooser chooser = new JFileChooser();
        	int returnVal;
        	if (currentPath.isEmpty())
        	{
        		returnVal = chooser.showOpenDialog(simulator);
        		currentPath = chooser.getCurrentDirectory().toString();
        	}
        	else
        	{
        		chooser.setCurrentDirectory(new File(currentPath));
        		returnVal = chooser.showOpenDialog(simulator);
        		currentPath = chooser.getCurrentDirectory().toString();
        	}
            if(returnVal == JFileChooser.APPROVE_OPTION) {
            	holoName = chooser.getSelectedFile().getName();
            	FileInputStream inputStreamForNN = new FileInputStream(chooser.getCurrentDirectory()+"\\"+holoName);
            	ObjectInputStream objectStreamForNN = new ObjectInputStream(inputStreamForNN);
            	Object obj = objectStreamForNN.readObject();
            	if(obj instanceof RecurrentNeuralNetwork)
            	{
            		//load this object which is a RNN into the current simulation brain object
            		((Evaluator_Environment)hologram).setPhenome((RecurrentNeuralNetwork)obj);
            	}
            	else
            	{
            		JOptionPane.showMessageDialog(simulator, "You must choose a file with extension .rnn generated from the Simbad_NEATAvoiderRobot simulator. This simulation will not work you must reload and try again.");
            	}
            	objectStreamForNN.close();
            	inputStreamForNN.close();
            	int returnValue = NNPiconode.showNetworkNoInterrupt((RecurrentNeuralNetwork)obj);
            }
        } catch (Exception e) {
        	JOptionPane.showMessageDialog(simulator, "You must choose a file with extension .rnn generated from the Simbad_NEATAvoiderRobot simulator. This simulation will not work you must reload and try again.");
            e.printStackTrace();
        }
        return hologram;
    }
}
