/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.app.NEAT_XOR;

import java.util.*;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.ext.evolution.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.*;
import simbad.piconode.core.node.RecurrentNeuralNetwork;
import simbad.piconode.visual.NNPiconode;

public class EvaluationOperator_NEATforXOR extends EvaluationOperator_Individual {

	public EvaluationOperator_NEATforXOR() {
	}

	public EvaluationOperator_NEATforXOR(String __name) {
		super(__name);
	}

	public void evaluate(Object o) 
	{
		super.evaluate(o);
		Individual_NEATIndividual individual = (Individual_NEATIndividual)o;
		
		try
		{
			individual.setRawFitness( this.computeFitness(individual));
		}
		catch ( java.lang.ClassCastException e )
		{
			Display.error(""+this.getClass().getName()+"::evaluate(.) - Class cast exception");
		}
	}
	
	private double computeFitness(Individual_NEATIndividual individual)
	{
		double outputDistance = 0.0;
		double distanceSum = 0.0;
		double adjustedSum = 0.0;
		boolean converged = false;
		
		//test data
		Vector<Double> input0 = new Vector<Double>(); //input values for node 0
		input0.add(0.0);
		input0.add(0.0);
		input0.add(1.0);
		input0.add(1.0);
		Vector<Double> input1 = new Vector<Double>(); //input values for node 1
		input1.add(0.0);
		input1.add(1.0);
		input1.add(0.0);
		input1.add(1.0);
		
		//expected output
		Vector<Double> output = new Vector<Double>(); //input values for node 1
		output.add(0.0);
		output.add(1.0);
		output.add(1.0);
		output.add(0.0);
		
		ArrayList<Double> inputValuesList = new ArrayList<Double>(); //input to the neural network
		
		RecurrentNeuralNetwork network = individual.createPhenotype();
		
		for(int epoch = 0; epoch < 500 && !converged; epoch++)
		{
			distanceSum = 0.0;
			adjustedSum = 0.0;
			for(int i = 0; i < input0.size(); i++)
			{
				inputValuesList.clear();
				inputValuesList.add(input0.get(i));
				inputValuesList.add(input1.get(i));
			
				network.step(inputValuesList);
			
				outputDistance = Math.abs(output.get(i) - network.getOutputNeuronAt(0).getOutputValue());
				distanceSum = distanceSum + outputDistance;
				if(outputDistance < 0.5)
				{
					adjustedSum += 1.0;
				}
			}

			if( adjustedSum > 3.9 )
			{
				Display.info("\n\n## ###################### ##");
				for(int i = 0; i < input0.size(); i++)
				{
					inputValuesList.clear();
					inputValuesList.add(input0.get(i));
					inputValuesList.add(input1.get(i));
			
					network.step(inputValuesList);
			
					double output2 = network.getOutputNeuronAt(0).getOutputValue();

					Display.info("Input: " + input0.get(i) + " " + input1.get(i) + " Output: " + output2 + "\r\n");
				}
				Display.info("\n\n## ###################### ##");
				converged = true;
				int returnValue = NNPiconode.showNetwork(network);
			}
		}
		
		network.updateGenotype(individual);
		((StatisticsOperator_NEAT)(individual.getTemplate().getPopulationStatisticsOperator())).logCollisionsDistanceEvals(individual.genomeID, 0,0,4);
		return Math.pow(4.0 - distanceSum, 2.0);
		
	}
}
