/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.app.Simbad_NEATAvoiderRobot;

import simbad.gui.Simbatch;
import simbad.picoevo.core.evolution.*;
import simbad.picoevo.ext.evolution.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.*;
import simbad.piconode.core.node.RecurrentNeuralNetwork;

public class EvaluationOperator_NEATAvoiderRobot extends
		EvaluationOperator_Individual {
	
	Evaluator_Maze environment;
	Evaluator_BusyHallway environment2;
	Evaluator_BusyRoom environment3;
	Simbatch simulator;
	Simbatch simulator2;
	Simbatch simulator3;
	int collisions;
	double distance;
	int evaluations;
	
	public EvaluationOperator_NEATAvoiderRobot() 
	{
		environment = new Evaluator_Maze();
		environment2 = new Evaluator_BusyHallway();
		environment3 = new Evaluator_BusyRoom();
		simulator = new Simbatch(environment, true);
		simulator2 = new Simbatch(environment2, true);
		simulator3 = new Simbatch(environment3, true);
		collisions = 0;
		distance = 0;
		evaluations = 0;
	}

	public EvaluationOperator_NEATAvoiderRobot(String name) {
		super(name);
		environment = new Evaluator_Maze();
		environment2 = new Evaluator_BusyHallway();
		environment3 = new Evaluator_BusyRoom();
		simulator = new Simbatch(environment, true);
		simulator2 = new Simbatch(environment2, true);
		simulator3 = new Simbatch(environment3, true);
		collisions = 0;
		distance = 0;
		evaluations = 0;
	}
	
	public EvaluationOperator_NEATAvoiderRobot(boolean show3d) 
	{
		environment = new Evaluator_Maze();
		environment2 = new Evaluator_BusyHallway();
		environment3 = new Evaluator_BusyRoom();
		simulator = new Simbatch(environment, show3d);
		simulator2 = new Simbatch(environment2, show3d);
		simulator3 = new Simbatch(environment3, show3d);
		collisions = 0;
		distance = 0;
		evaluations = 0;
	}

	public EvaluationOperator_NEATAvoiderRobot(String name, boolean show3d) {
		super(name);
		environment = new Evaluator_Maze();
		environment2 = new Evaluator_BusyHallway();
		environment3 = new Evaluator_BusyRoom();
		simulator = new Simbatch(environment, show3d);
		simulator2 = new Simbatch(environment2, show3d);
		simulator3 = new Simbatch(environment3, show3d);
		collisions = 0;
		distance = 0;
		evaluations = 0;
	}
	
	public void evaluate(Object o) 
	{
		super.evaluate(o);
		Individual_NEATIndividual individual = (Individual_NEATIndividual)o;
		
		try 
		{
			individual.setRawFitness( this.computeFitness(individual) );
		}
		catch ( java.lang.ClassCastException e )
		{
			Display.error(""+this.getClass().getName()+"::evaluate(.) - Class cast exception");
		}
	}

	private double computeFitness ( Individual_NEATIndividual individual )
	{
		double totalFitness = 0.0;

		RecurrentNeuralNetwork network = individual.createPhenotype();
		
		//TEST IN ENVIRONMENT ONE
		environment.setPhenome(network);
		environment.robot.resetEvaluation();
	    int i = 0;
	    while ( i < 30000 && (environment.isRunnable == true) ) 
	    {
	    	simulator.step();
	    	i++;
	    }
	    evaluations += i;
	    collisions += environment.robot.getCollisionCount();
	    distance += environment.robot.getDistanceToGoal();
	    totalFitness += environment.fitness;
	    network = environment.getPhenome();
	    environment.robot.resetCollisionCount();
	    environment.clearPhenome();
	    simulator.reset();

	    //TEST IN ENVIRONMENT TWO
		environment2.setPhenome(network);
		environment2.robot.resetEvaluation();
	    int j = 0;
	    while ( j < 30000 && (environment2.isRunnable == true) ) 
	    {
	    	simulator2.step();
	    	j++;
	    }
	    evaluations += j;
	    collisions += environment.robot.getCollisionCount();
	    distance += environment.robot.getDistanceToGoal();
	    totalFitness += environment2.fitness;
	    network = environment2.getPhenome();
	    environment2.robot.resetCollisionCount();
	    environment2.clearPhenome();
	    simulator2.reset();
	    
	    //TEST IN ENVIRONMENT TWO
		environment3.setPhenome(network);
		environment3.robot.resetEvaluation();
	    int k = 0;
	    while ( k < 30000 && (environment3.isRunnable == true) ) 
	    {
	    	simulator3.step();
	    	k++;
	    }
	    evaluations += k;
	    collisions += environment.robot.getCollisionCount();
	    distance += environment.robot.getDistanceToGoal();
	    ((StatisticsOperator_NEAT)(individual.getTemplate().getPopulationStatisticsOperator())).logCollisionsDistanceEvals(individual.genomeID, collisions,distance,evaluations);
	    totalFitness += environment3.fitness;
	    network = environment3.getPhenome();
	    environment3.robot.resetCollisionCount();
	    environment3.clearPhenome();
	    simulator3.reset();
	    
	    collisions = 0;
	    evaluations = 0;
	    distance = 0;
	    network.updateGenotype(individual);
	    //Take the average of the fitness value across the three trials
	    totalFitness = totalFitness/3.0;
	    return totalFitness;
	}

}
