/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.app.Simbad_SDNEATAvoiderRobot;

import simbad.sim.StraightGoalLoopAgent;
import simbad.sim.Wall;
import javax.vecmath.Vector3d;
import simbad.piconode.core.node.*;

public class Evaluator_BusyRoom extends Evaluator_Environment {

	public Evaluator_BusyRoom()
	{
		//perimeter Walls
        Wall north = new Wall(new Vector3d(8, 0, 0), 20, 1, this);
        north.rotate90(1);
        add(north);
        Wall south = new Wall(new Vector3d(-8, 0, 0), 20, 1, this);
        south.rotate90(1);
        add(south);
        Wall east = new Wall(new Vector3d(0, 0, 10), 16, 1, this);
        add(east);
        Wall west = new Wall(new Vector3d(0, 0, -10), 16, 1, this);
        add(west);
		
		//middle walls
		Wall w1 = new Wall(new Vector3d(-5,0,0), 6, 1, this);
		add(w1);
		Wall w2 = new Wall(new Vector3d(5,0,0), 6, 1, this);
		add(w2);
		
		robot = new NeuralAgent(new Vector3d(0,0,7), new Vector3d(0,0,-6), "NeuralAgent", this);
		add(robot);
		
		//top 2 agents
		StraightGoalLoopAgent Robot2 = new StraightGoalLoopAgent(new Vector3d(-3,0,-1), new Vector3d(3,0,-3), "Top1");
		add(Robot2);
		StraightGoalLoopAgent Robot3 = new StraightGoalLoopAgent(new Vector3d(-6,0,-3), new Vector3d(6,0,-1), "Top2");
		add(Robot3);
		
		//bottom 2 agents
		StraightGoalLoopAgent Robot6 = new StraightGoalLoopAgent(new Vector3d(6,0,-1), new Vector3d(-6,0,-3), "Bottom1");
		add(Robot6);
		StraightGoalLoopAgent Robot7 = new StraightGoalLoopAgent(new Vector3d(3,0,-3), new Vector3d(-3,0,-1), "Bottom2");
		add(Robot7);
		
		//bottom 2 agents
		StraightGoalLoopAgent Robot9 = new StraightGoalLoopAgent(new Vector3d(-1,0,-6), new Vector3d(4,0,6), "TopRight");
		add(Robot9);
		StraightGoalLoopAgent Robot10 = new StraightGoalLoopAgent(new Vector3d(1,0,-6), new Vector3d(-4,0,6), "BottomRight");
		add(Robot10);
	}
	
	/** This MUST be run before running simulations with this evaluator*/
    public void setPhenome (RecurrentNeuralNetwork givenPhenome)
    {
        this.phenome = givenPhenome;
    }
    
    public RecurrentNeuralNetwork getPhenome()
    {
    	return this.phenome;
    }
    
    public void clearPhenome()
    {
    	this.phenome = null;
    }
}
