/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.evolution;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.*;

public class EvolveOperator_Population_NEATPopulation extends EvolveOperator {

	public EvolveOperator_Population_NEATPopulation() {
		super("Unnamed NEAT population evolve operator");
	}
	
	public EvolveOperator_Population_NEATPopulation(String name) {
		super(name);
	}

	@Override
	public void evolve(Object o) {
		evolve(o,false);
	}

	public void evolve(Object o, boolean display) {
		// Perform main epoch work here.
		Population_NEATPopulation population = (Population_NEATPopulation)o;
		
		if ( display == true)
			Display.info("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
			Display.info("\n# Generation no." + population.getGenerationNumber());
			Display.info("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
			
		population.performEvaluation(); //evaluate population by calling individual_evaluation_function for each individual

		// statistics
		if ( display == true)
		{
			//Inhibits displaying the neural network structure of the best agent after each generation
			population.displayStatisticsNoGUI();
			//displays the neural network structure of the best agent after each generation
			//population.displayStatistics();
		}
		
		population.getTemplate().getSelectionOperator().performSelection(population); //create new population

		// [DEBUG] Memory check 
		if ( display == true)
			Display.memorycheck();
	}
	
}
