/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.evolution;

import java.util.ArrayList;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.core.representation.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.*;

public class InitialisationOperator_Individual_NEAT extends
		InitialisationOperator_Individual {

	public InitialisationOperator_Individual_NEAT() {
	}

	public InitialisationOperator_Individual_NEAT(String __name) {
		super(__name);
	}

	@Override
	/**This code will create and initialise a minimal genome where there are output + input
	 * neurons and each input neuron is connected to each output neuron.
	 * REQUIRES: Passed in object to have number of inputs and outputs already set. */
	public void initialise(Individual object) {
		try {
			Individual_NEATIndividual individual = (Individual_NEATIndividual)object;
			
			ArrayList operatorList = new ArrayList();
			VariationOperator_Element_NEAT_Gene op = new VariationOperator_Element_NEAT_Gene("initialization operator",
																							((ParameterSet_Evolution_NEAT)individual.getTemplate()).getProbabilityRateReplaced(),
																							((ParameterSet_Evolution_NEAT)individual.getTemplate()).getProbabilityRateReplaced(),
																							((ParameterSet_Evolution_NEAT)individual.getTemplate()).getMaxWeightPerturbation(),
																							((ParameterSet_Evolution_NEAT)individual.getTemplate()).getActivationMutationRate(),
																							((ParameterSet_Evolution_NEAT)individual.getTemplate()).getMaxActivationPerturbation());
			double inputRowsSlice = 0.8/((double)(individual.numInputs()));
			double ouputRowsSlice = 1.0/((double)(individual.numOutputs()));
			int neuronTypeInput = 1; //for readability, a neuron type of 1 means the neuron is an input node
			int neuronTypeOutput = 2; //for readability, a neuron type of 2 means the neuron is an output node
			int neuronTypeBias = 4; //for readability, a neuron type of 4 means the neuron is a bias node.
			double defaultActivationResponse = 1.0;
			boolean defaultRecurrent = false;
			int innovationID = 0; //this MUST be set correctly somewhere else
			
			//input slice
			for (int i = 0; i < individual.numInputs(); i++)
			{
				individual.addNeuron(new Element_NEAT_NGene("neuron-"+i, individual,
															operatorList,
															innovationID,
															defaultRecurrent,
															neuronTypeInput,
															defaultActivationResponse,
															0.0,
															0.1+i*inputRowsSlice,
															i));
				innovationID++;
			}
			
			//NOTE: This will change how other classes function too:
			//VariationOperator_Individual_NEAT_MutationAddLink
			
			//Create and add a Bias node
			//putting this line in for completeness but commenting out until I know the behaviour
			/*individual.addNeuron(new Element_NEAT_NGene("neuron"+i, individual,
								operatorList,
								innovationID,
								defaultRecurrent,
								neuronTypeBias,
								defaultActivationResponse,
								0.0,
								0.9,
								individual.numInputs()));*/
			
			//IMPORTANT if you add in the bias node above you MUST add +1 to every reference to individual.numInputs() below.
			//output slice
			for (int i = 0; i < individual.numOutputs(); i++)
			{
				individual.addNeuron(new Element_NEAT_NGene("neuron-"+(i+individual.numInputs()), individual,
															operatorList,
															innovationID,
															defaultRecurrent,
															neuronTypeOutput,
															defaultActivationResponse,
															0.0,
															0.1+i*inputRowsSlice,
															i+individual.numInputs()));
				innovationID++;
			}
			
			//IMPORTANT if you add in the bias node above you MUST add +1 to every reference to individual.numInputs() below.
			//create the link genese, connect each input neuron to each output neuron
			//and assign a random weight -1 < w < 1 (by performing variations at the end of this function)
			for (int i = 0; i < individual.numInputs(); i++)
			{
				for (int o = 0; o < individual.numOutputs(); o++)
				{
					individual.addLink(new Element_NEAT_LGene("link-"+i+"->"+o, individual,
															  operatorList,
															  innovationID,
															  defaultRecurrent,
															  individual.getNeuronAt(i).getNeuronID(),
															  individual.getNeuronAt(individual.numInputs()+o).getNeuronID(),
															  1.0));
					innovationID++;
				}
			}
			
			//Now run through the list of elements in this individual and fire all the initialisation operators
			for (int i = 0; i < individual.getElementListSize(); i++)
			{
				individual.getElementAt(i).performInitialisation();
			}
		}
		catch ( java.lang.ClassCastException e )
		{
			Display.error(""+this.getClass().getName()+"::initialise(.) - Class cast exception");
		}

	}

}
