/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.evolution;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.core.representation.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.*;

public class InitialisationOperator_Population_NEATPopulation extends
		InitialisationOperator_Population {

	public InitialisationOperator_Population_NEATPopulation() {
	}

	public InitialisationOperator_Population_NEATPopulation(String __name) {
		super(__name);
	}

	@Override
	public void initialise(Population object) {
		try {
			int defaultNetworkDepth = 1;
			int defaultGenomeSpecies = 0;
			
			//Create the population of minimal genomes
			Population_NEATPopulation population = (Population_NEATPopulation)object;
			for (int i = 0; i != population.getTemplate().getInitPopSize(); i++)
			{
				Individual_NEATIndividual individual = (new Individual_NEATIndividual("individual-"+i,
																					  population,
																					  i,
																					  defaultNetworkDepth,
																					  ((ParameterSet_Evolution_NEAT)population.getTemplate()).getInitialGenomeInputs(),
																					  ((ParameterSet_Evolution_NEAT)population.getTemplate()).getInitialGenomeOutputs(),
																					  defaultGenomeSpecies));
				individual.performInitialisation();
				population.registerIndividual(individual);
			}
			//Create the innovation list:
			//First create a minimal genome
			Individual_NEATIndividual initIndividual = (new Individual_NEATIndividual("individual-for-innovation-list",
					  								population,
					  								1,
					  								defaultNetworkDepth,
					  								((ParameterSet_Evolution_NEAT)population.getTemplate()).getInitialGenomeInputs(),
					  								((ParameterSet_Evolution_NEAT)population.getTemplate()).getInitialGenomeOutputs(),
					  								defaultGenomeSpecies));
			initIndividual.performInitialisation();
			population.innovationList.initializeInnovationList(initIndividual.getLinks(), initIndividual.getNeurons());
			//May have to create a network depth lookup table here (as per C++ NEAT implementation)
			//I think it is not required in this implementation
		}
		catch ( java.lang.ClassCastException e )
		{
			Display.error(""+this.getClass().getName()+"::initialise(.) - Class cast exception");
		}
	}

}
