/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.evolution;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.toolbox.*;

public class ParameterSet_Evolution_NEAT extends ParameterSet {

	//Parameters to set for the Compatibility Score between species
	//Located in Individial_NEATIndividual.getCompatibilityScore
	protected double disjointModifier = 1;
	protected double excessModifier = 1;
	protected double matchedModifier = 0.4;
	
	//Parameters to set for adjusting the compatibility threshold
	//Located in  Population_NEATPopulation_Species.speciate
	protected double initialCompatibilityThreshold = 0.10; //This value must NEVER start at 0.
	protected double thresholdIncrement = 0.01;
	protected int maximumNumberOfSpecies = 1;
	
	//Parameters to set for adjusting the fitness level of each individual
	//Located in Population_NEATPopulation_Species.AdjustFitnesses	
	protected int youngBonusAgeThreshold;
	protected double youngFitnessBonus;
	protected int oldAgeThreshold;
	protected double oldAgePenalty;
	
	//Parameters to set for adjusting the survival rate for spawning
	//Called in Population_NEATPopulation_SpeciesList::Species::Spawn
	protected double survivalRate; //number of population to survive each epoch (0.2 = 20%)
	
	//Parameters to control the level of variation 
	//Located in VariationOperator_Element_NEAT_Gene 
	//Called in InitialisationOperator_Individual_NEATIndividual during individual initialisation
	//Called in SelectionOperator_NEAT during mutation 
	protected double probabilityRateReplaced;
	protected double maxWeightPerturbation;
	protected double activationMutationRate;
	protected double maxActivationPerturbation;
	
	//Parameters to control the default genome that is created
	//Called in InitialisationOperator_Population_NEATPopulation during population initialisation
	protected int initialGenomeInputs = 1;
	protected int initialGenomeOutputs = 1;

	//Parameters to control the population
	//Called in SelectionOperator_NEAT
	protected int numGensAllowedNoImprovement = 1;
	protected double crossoverRate = 1.0;
	protected int maximumNumberOfNeurons = 10000; //limits adding more neurons to a spawned child
	
	//Parameters to control the amount of variation when adding links to a genome
	//Used by VariationOperator_Individual_NEAT_MutationAddLink
	//Used by VariationOperator_Individual_NEAT_MutationAddNode
	//Called in SelectionOperator_NEAT 
	protected double mutationRate;
	protected double chanceAddNode; //chance of adding a node to a child
	protected double chanceAddLink; //chance of adding a link to a child
	protected double chanceOfLoopedLink; //chance of adding a recurrent link
	protected int numTrysToFindLoop; //number of attempts to find a non looped link
	protected int numTrysToAddLink; //number of attempts to add a link to a child
	protected int numTrysToFindOldLink;
	
	protected String experimentPath;
	
	public void displayInformation() {
		Display.info("\n##Parameter Setup##");
		Display.info("- Maximisation?             : " + this.getOptimisationFlag());
		Display.info("- Pop. Initialisation Op.   : " + this.getPopulationInitialisationOperator().getName());
		Display.info("- Ind. Initialisation Op.   : " + this.getIndividualInitialisationOperator().getName());
		Display.info("- max number of generations : " + this.getGenerations());
		Display.info("- initial population size   : " + this.getInitPopSize());
	}
	
	public ParameterSet_Evolution_NEAT() {
	}

	public ParameterSet_Evolution_NEAT(String __filename) {
		super(__filename);
	}
	
	public void setDisjointModifier(double m){
		this.disjointModifier = m;
	}
	
	public double getDisjointModifier(){
		return disjointModifier;
	}
	
	public void setExcessModifier(double e){
		this.excessModifier = e;
	}
	
	public double getExcessModifier(){
		return excessModifier;
	}
	
	public void setMatchedModifier(double m){
		this.matchedModifier = m;
	}
	
	public double getMatchedModifier(){
		return matchedModifier;
	}
	
	public double getInitialCompatibilityThreshold() {
		return initialCompatibilityThreshold;
	}

	public void setInitialCompatibilityThreshold(
			double initialCompatibilityThreshold) {
		this.initialCompatibilityThreshold = initialCompatibilityThreshold;
	}

	public void setThresholdIncrement(double t){
		this.thresholdIncrement = t;
	}
	
	public double getThresholdIncrement(){
		return thresholdIncrement;
	}

	public int getMaximumNumberOfSpecies() {
		return maximumNumberOfSpecies;
	}

	public void setMaximumNumberOfSpecies(int maximumNumberOfSpecies) {
		this.maximumNumberOfSpecies = maximumNumberOfSpecies;
	}

	public int getYoungBonusAgeThreshold() {
		return youngBonusAgeThreshold;
	}

	public void setYoungBonusAgeThreshold(int youngBonusAgeThreshold) {
		this.youngBonusAgeThreshold = youngBonusAgeThreshold;
	}

	public double getYoungFitnessBonus() {
		return youngFitnessBonus;
	}

	public void setYoungFitnessBonus(double youngFitnessBonus) {
		this.youngFitnessBonus = youngFitnessBonus;
	}

	public int getOldAgeThreshold() {
		return oldAgeThreshold;
	}

	public void setOldAgeThreshold(int oldAgeThreshold) {
		this.oldAgeThreshold = oldAgeThreshold;
	}

	public double getOldAgePenalty() {
		return oldAgePenalty;
	}

	public void setOldAgePenalty(double oldAgePenalty) {
		this.oldAgePenalty = oldAgePenalty;
	}

	public double getProbabilityRateReplaced() {
		return probabilityRateReplaced;
	}

	public void setProbabilityRateReplaced(double probabilityRateReplaced) {
		this.probabilityRateReplaced = probabilityRateReplaced;
	}

	public double getMaxWeightPerturbation() {
		return maxWeightPerturbation;
	}

	public void setMaxWeightPerturbation(double maxWeightPerturbation) {
		this.maxWeightPerturbation = maxWeightPerturbation;
	}

	public double getActivationMutationRate() {
		return activationMutationRate;
	}

	public void setActivationMutationRate(double activationMutationRate) {
		this.activationMutationRate = activationMutationRate;
	}

	public double getMaxActivationPerturbation() {
		return maxActivationPerturbation;
	}

	public void setMaxActivationPerturbation(double maxActivationPerturbation) {
		this.maxActivationPerturbation = maxActivationPerturbation;
	}

	public int getInitialGenomeInputs() {
		return initialGenomeInputs;
	}

	public void setInitialGenomeInputs(int initialGenomeInputs) {
		this.initialGenomeInputs = initialGenomeInputs;
	}

	public int getInitialGenomeOutputs() {
		return initialGenomeOutputs;
	}

	public void setInitialGenomeOutputs(int initialGenomeOutputs) {
		this.initialGenomeOutputs = initialGenomeOutputs;
	}

	public int getNumGensAllowedNoImprovement() {
		return numGensAllowedNoImprovement;
	}

	public void setNumGensAllowedNoImprovement(int numGensAllowedNoImprovement) {
		this.numGensAllowedNoImprovement = numGensAllowedNoImprovement;
	}

	public double getMutationRate() {
		return mutationRate;
	}

	public void setMutationRate(double mutationRate) {
		this.mutationRate = mutationRate;
	}

	public double getChanceOfLoopedLink() {
		return chanceOfLoopedLink;
	}

	public void setChanceOfLoopedLink(double chanceOfLoopedLink) {
		this.chanceOfLoopedLink = chanceOfLoopedLink;
	}

	public int getNumTrysToFindLoop() {
		return numTrysToFindLoop;
	}

	public void setNumTrysToFindLoop(int numTrysToFindLoop) {
		this.numTrysToFindLoop = numTrysToFindLoop;
	}

	public int getNumTrysToAddLink() {
		return numTrysToAddLink;
	}

	public void setNumTrysToAddLink(int numTrysToAddLink) {
		this.numTrysToAddLink = numTrysToAddLink;
	}

	public int getNumTrysToFindOldLink() {
		return numTrysToFindOldLink;
	}

	public void setNumTrysToFindOldLink(int numTrysToFindOldLink) {
		this.numTrysToFindOldLink = numTrysToFindOldLink;
	}

	public double getSurvivalRate() {
		return survivalRate;
	}

	public void setSurvivalRate(double survivalRate) {
		this.survivalRate = survivalRate;
	}

	public double getCrossoverRate() {
		return crossoverRate;
	}

	public void setCrossoverRate(double crossoverRate) {
		this.crossoverRate = crossoverRate;
	}

	public int getMaximumNumberOfNeurons() {
		return maximumNumberOfNeurons;
	}

	public void setMaximumNumberOfNeurons(int maximumNumberOfNeurons) {
		this.maximumNumberOfNeurons = maximumNumberOfNeurons;
	}

	public double getChanceAddNode() {
		return chanceAddNode;
	}

	public void setChanceAddNode(double chanceAddNode) {
		this.chanceAddNode = chanceAddNode;
	}

	public double getChanceAddLink() {
		return chanceAddLink;
	}

	public void setChanceAddLink(double chanceAddLink) {
		this.chanceAddLink = chanceAddLink;
	}

	public String getExperimentPath() {
		return experimentPath;
	}

	public void setExperimentPath(String experimentPath) {
		this.experimentPath = experimentPath;
	}
}
