/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.evolution;

import java.io.*;
import java.util.ArrayList;
import java.util.Vector;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.*;
import simbad.piconode.visual.NNPiconode;
import simbad.piconode.core.node.RecurrentNeuralNetwork;

public class StatisticsOperator_NEAT extends StatisticsOperator {

	public StatisticsOperator_NEAT() { super("unnamed statistics operator"); }
	public StatisticsOperator_NEAT(String __name) { super(__name); }
	public double bestFitnessEver = 0.0;
	public String bestGenomeEver = new String("none");
	
	//**Population statistics**
	//Species statistics
	public Vector<Vector<Integer>> speciesGenerationsVsSize = new Vector<Vector<Integer>>();
	Vector<Integer> speciesRecord = null; //element of the above vector.
	//Innovation statistics
	public Vector<Integer> numberOfNewInnovations = new Vector<Integer>();
	public Vector<Integer> numberOfInnovations = new Vector<Integer>();
	//Connection statistics
	public Vector<Integer> mostConnections = new Vector<Integer>();
	public int cMostConnections;
	public Vector<Integer> leastConnections = new Vector<Integer>();
	public int cLeastConnections;
	public Vector<Double> meanConnections = new Vector<Double>();
	public double cMeanConnections;
	public Vector<Double> medianConnections = new Vector<Double>();
	public double cMedianConnections;
	public Vector<Integer> connectionsInBestEver = new Vector<Integer>();
	public int cConnectionsInBestEver;
	public Vector<Integer> connectionsInBestCurrent = new Vector<Integer>();
	public int cConnectionsInBestCurrent;
	//Node statistics
	public Vector<Integer> mostNodes = new Vector<Integer>();
	public int cMostNodes;
	public Vector<Integer> leastNodes = new Vector<Integer>();
	public int cLeastNodes;
	public Vector<Double> meanNodes = new Vector<Double>();
	public double cMeanNodes;
	public Vector<Double> medianNodes = new Vector<Double>();
	public double cMedianNodes;
	public Vector<Integer> nodesInBestEver = new Vector<Integer>();
	public int cNodesInBestEver;
	public Vector<Integer> nodesInBestCurrent = new Vector<Integer>();
	public int cNodesInBestCurrent;
	
	//**Experiment statistics**
	//Experiment fitness
	public Vector<Double> genFitnessMax = new Vector<Double>();
	public double cGenFitnessMax;
	public Vector<Double> genFitnessMin = new Vector<Double>();
	public double cGenFitnessMin;
	public Vector<Double> genFitnessMean = new Vector<Double>();
	public double cGenFitnessMean;
	public Vector<Double> genFitnessMedian = new Vector<Double>();
	public double cGenFitnessMedian;
	public Vector<Double> genFitnessInBestEver = new Vector<Double>();
	public double cGenFitnessInBestEver;
	public Vector<Double> genFitnessInBestCurrent = new Vector<Double>();
	public double cGenFitnessInBestCurrent;
	
	public Vector<Integer> gIDGenerationLog = new Vector<Integer>(); //for dumping the gid's for collision and distance logs
	
	//Experiment collisions
	public Vector<Integer> genCollisionsMax = new Vector<Integer>();
	public Vector<Integer> genCollisionsMin = new Vector<Integer>();
	public Vector<Integer> genCollisionsMedian = new Vector<Integer>();
	public Vector<Double> genCollisionsMean = new Vector<Double>();
	public Vector<Integer> genCollisionsBestEver = new Vector<Integer>();
	public Vector<Integer> genCollisionsBestCurrent = new Vector<Integer>();
	public Vector<Integer> generationCollisionsLog = new Vector<Integer>(); //for dumping the collision totals each generation
	
	//Experiment distance to goal
	public Vector<Double> genDistanceToGoalMax = new Vector<Double>();
	public Vector<Double> genDistanceToGoalMin = new Vector<Double>();
	public Vector<Double> genDistanceToGoalMedian = new Vector<Double>();
	public Vector<Double> genDistanceToGoalMean = new Vector<Double>();
	public Vector<Double> genDistanceToGoalBestEver = new Vector<Double>();
	public Vector<Double> genDistanceToGoalBestCurrent = new Vector<Double>();
	public Vector<Double> generationDistanceLog = new Vector<Double>(); //for dumping the distance to goal values to log
	
	//Experiment evaluations
	public Vector<Integer> genNumEvaluationsMax = new Vector<Integer>();
	public Vector<Integer> genNumEvaluationsMin = new Vector<Integer>();
	public Vector<Integer> genNumEvaluationsMedian = new Vector<Integer>();
	public Vector<Double> genNumEvaluationsMean = new Vector<Double>();
	public Vector<Integer> genNumEvaluationsBestEver = new Vector<Integer>();
	public Vector<Integer> genNumEvaluationsBestCurrent = new Vector<Integer>();
	public Vector<Integer> generationEvaluationLog = new Vector<Integer>();
	
	public void displayStatistics ( Object o )
	{
		Population_NEATPopulation p = (Population_NEATPopulation)o;
		
		Display.info(p.getPopulationSize() + " individuals.");
		Display.info("Best individual currently is known as Genome:" + p.getRawBest().genomeID);
		Display.info("Best individual ever is " + bestGenomeEver);
		Display.info("With the raw fitness: " + bestFitnessEver);
		int returnValue = NNPiconode.showNetworkNoInterrupt(p.getRawBest().createPhenotype());
	}
	
	public void displayStatisticsNoGUI ( Object o )
	{
		Population_NEATPopulation p = (Population_NEATPopulation)o;
		
		Display.info(p.getPopulationSize() + " individuals.");
		Display.info("Best individual currently is known as Genome:" + p.getRawBest().genomeID);
		Display.info("Best individual ever is " + bestGenomeEver);
		Display.info("With the raw fitness: " + bestFitnessEver);
	}
	
	public void logStatistics ( Object o, String filename )
	{
		logExperiment((Population_NEATPopulation)o);	
	}

	public void logGeneration ( Population_NEATPopulation p, String path )
	{
		String basePath = ((ParameterSet_Evolution_NEAT)p.getTemplate()).getExperimentPath();
		basePath += path + "/";
		String fileName = new String();
		Individual_NEATIndividual rawBestGenome = p.getRawBest();
		
		try {
				//Compute this generations statistics
				computeStatistics(p);
			
				//create the log directories if they don't exist 
				File oPath = new File(basePath);
				oPath.mkdirs();
				
				//Write a file with the description of this generations best genome
				fileName = "GenerationChampionDescription" + rawBestGenome.genomeID + ".txt";
				BufferedWriter outputFile = new BufferedWriter(new FileWriter(basePath+fileName, true));
				outputFile.write(rawBestGenome.toString());
				outputFile.close();
				
				//Write the actual object for this generations best genome
				fileName = "GenerationChampionObject" + rawBestGenome.genomeID + ".rnn";
				FileOutputStream outputFileNN = new FileOutputStream(basePath+fileName);
				ObjectOutputStream outputObjectNN = new ObjectOutputStream(outputFileNN);
				RecurrentNeuralNetwork tRNN = rawBestGenome.createPhenotype();
				outputObjectNN.writeObject(tRNN);
				outputObjectNN.close();
				outputFileNN.close();
				
				//Write the objects for each species champion this generation
				//Along with the top contenders in each species
				for(int spec = 0; spec < p.speciesList.numOfSpecies(); spec++)
				{
					//Write the actual object for this generations best genome
					fileName = "Index-" + spec + "-SpeciesNumber-" + p.speciesList.getSpeciesAt(spec).speciesID + "-SpeciesChampionObject-" + ".rnn";
					FileOutputStream outputFileSpeciesNN = new FileOutputStream(basePath+fileName);
					ObjectOutputStream outputObjectSpeciesNN = new ObjectOutputStream(outputFileSpeciesNN);
					RecurrentNeuralNetwork sRNN = p.speciesList.getSpeciesAt(spec).alphaMember.createPhenotype();
					//Calculate how many genomes in the top 25% of this species
					int top25 = p.speciesList.getSpeciesAt(spec).members.size()/4;
					//Store the top 25% to disk.
					for(int gen = 0; gen < top25; gen++)
					{
						//Write the actual object for this generations best genome
						String subFileName = new String();
						subFileName = "Index-" + spec + "-SpeciesNumber-" + p.speciesList.getSpeciesAt(spec).speciesID + "-Place-" + gen + "-SpeciesRunnerUpObject-" + ".rnn";
						FileOutputStream outputFileSpeciesSubNN = new FileOutputStream(basePath+subFileName);
						ObjectOutputStream outputObjectSpeciesSubNN = new ObjectOutputStream(outputFileSpeciesSubNN);
						RecurrentNeuralNetwork subRNN = p.speciesList.getSpeciesAt(spec).members.elementAt(gen).createPhenotype();
						outputObjectSpeciesSubNN.writeObject(subRNN);
						outputObjectSpeciesSubNN.close();
						outputFileSpeciesSubNN.close();
					}
					outputObjectSpeciesNN.writeObject(sRNN);
					outputObjectSpeciesNN.close();
					outputFileSpeciesNN.close();
				}
				
				//Dump the standing champion information to a file
				fileName = "GenerationChampion.txt";
				BufferedWriter outputFileGenerationChampion = new BufferedWriter(new FileWriter(basePath+fileName, true));
				outputFileGenerationChampion.write("Best individual this generation is " + "Generation" + p.getGenerationNumber() + "-> GenomeID:" + rawBestGenome.genomeID + "\r\n");
				outputFileGenerationChampion.write("With the raw fitness: " + genFitnessInBestCurrent.elementAt(genFitnessInBestCurrent.size()-1) + "\r\n");
				outputFileGenerationChampion.write("It contains this many connections: " + connectionsInBestCurrent.elementAt(connectionsInBestCurrent.size()-1) + "\r\n");
				outputFileGenerationChampion.write("It contains this many neurons: " + nodesInBestCurrent.elementAt(nodesInBestCurrent.size()-1) + "\r\n");
				outputFileGenerationChampion.write("In its last simulation it had this many collisions: " + genCollisionsBestCurrent.elementAt(genCollisionsBestCurrent.size()-1) + "\r\n");
				outputFileGenerationChampion.write("In its last simulation the total accumulated distance to goal was: " + genDistanceToGoalBestCurrent.elementAt(genDistanceToGoalBestCurrent.size()-1) + "\r\n");
				outputFileGenerationChampion.write("In its last simulation the total number of simulation steps was: " + genNumEvaluationsBestCurrent.elementAt(genNumEvaluationsBestCurrent.size()-1) + "\r\n");
				outputFileGenerationChampion.close();
				
				//Dump the standing champion information to a file
				fileName = "ExperimentChampion.txt";
				BufferedWriter outputFileStandingChampion = new BufferedWriter(new FileWriter(basePath+fileName, true));
				outputFileStandingChampion.write("Best individual ever is " + bestGenomeEver + "\r\n");
				outputFileStandingChampion.write("With the raw fitness: " + bestFitnessEver + "\r\n");
				outputFileStandingChampion.write("It contains this many connections: " + connectionsInBestEver.elementAt(connectionsInBestEver.size()-1) + "\r\n");
				outputFileStandingChampion.write("It contains this many neurons: " + nodesInBestEver.elementAt(nodesInBestEver.size()-1) + "\r\n");
				outputFileStandingChampion.write("In its last simulation it had this many collisions: " + genCollisionsBestEver.elementAt(genCollisionsBestEver.size()-1) + "\r\n");
				outputFileStandingChampion.write("In its last simulation the total accumulated distance to goal was: " + genDistanceToGoalBestEver.elementAt(genDistanceToGoalBestEver.size()-1) + "\r\n");
				outputFileStandingChampion.write("In its last simulation the total number of simulation steps was: " + genNumEvaluationsBestEver.elementAt(genNumEvaluationsBestEver.size()-1) + "\r\n");
				outputFileStandingChampion.close();
				
				//print all genomes all generations to a file.
				fileName = "GenomeDetailedPopulation.txt";
				BufferedWriter outputFileGenomes = new BufferedWriter(new FileWriter(basePath+fileName, true));
				for(int i = 0; i < p.populationSize; i++)
				{
					outputFileGenomes.write(((Individual_NEATIndividual)p.getIndividual(i)).toString());
				}
				outputFileGenomes.close();
				
				//Dump the current species list to a file
				fileName = "SpeciesList.txt";
				BufferedWriter outputFileSpeciesList = new BufferedWriter(new FileWriter(basePath+fileName, true));
				outputFileSpeciesList.write(p.speciesList.toString());
				outputFileSpeciesList.close();
				
				//Dump species list statistics
				fileName = "SpeciesListStats.txt";
				BufferedWriter outputFileSpeciesListStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
				int largestRecord = 0;
				//find the largest record
				for(int i = 0; i < speciesGenerationsVsSize.size(); i++)
				{
					if( speciesGenerationsVsSize.elementAt(i).size() > largestRecord)
					{
						largestRecord = speciesGenerationsVsSize.elementAt(i).size();
					}
				}
				//write the header for the species stats
				StringBuffer speciesHeader = new StringBuffer();
				speciesHeader.append("Generation vs Species Size,");
				for(int i = 0; i < largestRecord; i++)
				{
					speciesHeader.append(i);
					if(i < largestRecord-1)
						speciesHeader.append(", ");
				}
				speciesHeader.append("\r\n");
				outputFileSpeciesListStats.write(speciesHeader.toString());
				//write the data for the species stats
				for(int i = 0; i < speciesGenerationsVsSize.size(); i++)
				{
					StringBuffer speciesRecord = new StringBuffer();
					speciesRecord.append("Generation ");
					speciesRecord.append(i);
					speciesRecord.append(", ");
					for(int j = 0; j < speciesGenerationsVsSize.elementAt(i).size(); j++)
					{
						speciesRecord.append(speciesGenerationsVsSize.elementAt(i).elementAt(j));
						if(j < speciesGenerationsVsSize.elementAt(i).size()-1)
							speciesRecord.append(", ");
					}
					speciesRecord.append("\r\n");
					outputFileSpeciesListStats.write(speciesRecord.toString());
				}
				outputFileSpeciesListStats.close();
				
				//Dump the innovation list statistics to a file
				fileName = "InnovationStats.txt";
				BufferedWriter outputFileInnovationStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
		 		outputFileInnovationStats.write("GenerationVSInnovations, Number of Innovations, Number of New Innovations \r\n");
		 		for(int i = 0; i < numberOfInnovations.size(); i++)
	    		{
		 			StringBuffer innovationStats = new StringBuffer();
		 			innovationStats.append(i);
		 			innovationStats.append(", ");
		 			innovationStats.append(numberOfInnovations.elementAt(i));
		 			innovationStats.append(", ");
		 			innovationStats.append(numberOfNewInnovations.elementAt(i));
		 			innovationStats.append("\r\n");
		 			outputFileInnovationStats.write(innovationStats.toString());
	    		}
				outputFileInnovationStats.close();
				
				//Dump the current innovation list to a file
				fileName = "InnovationList.txt";
				BufferedWriter outputFileInnovationList = new BufferedWriter(new FileWriter(basePath+fileName, true));
				outputFileInnovationList.write(p.innovationList.toString());
				outputFileInnovationList.close();
				
				//Dump the current generation connection statistics
				fileName = "ConnectionStats.txt";
				BufferedWriter outputFileConnectionStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
		 		outputFileConnectionStats.write("GenerationVSConnections, MostConnections, LeastConnections, MeanConnections, MedianConnections, BestCurrent, BestEver \r\n");
		 		for(int i = 0; i < mostConnections.size(); i++)
	    		{
		 			StringBuffer connectionStats = new StringBuffer();
					connectionStats.append(i);
					connectionStats.append(", ");
					connectionStats.append(mostConnections.elementAt(i));
					connectionStats.append(", ");
					connectionStats.append(leastConnections.elementAt(i));
					connectionStats.append(", ");
					connectionStats.append(meanConnections.elementAt(i));
					connectionStats.append(", ");
					connectionStats.append(medianConnections.elementAt(i));
					connectionStats.append(", ");
					connectionStats.append(connectionsInBestCurrent.elementAt(i));
					connectionStats.append(", ");
					connectionStats.append(connectionsInBestCurrent.elementAt(i));
					connectionStats.append("\r\n");
		 			outputFileConnectionStats.write(connectionStats.toString());
	    		}
				outputFileConnectionStats.close();
				
				//Dump the current generation node statistics
				fileName = "NodeStats.txt";
				BufferedWriter outputFileNodeStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
		 		outputFileNodeStats.write("GenerationVSNodes, MostNodes, LeastNodes, MeanNodes, MedianNodes, BestCurrent, BestEver \r\n");
		 		for(int i = 0; i < mostNodes.size(); i++)
	    		{
		 			StringBuffer nodeStats = new StringBuffer();
		 			nodeStats.append(i);
		 			nodeStats.append(", ");
		 			nodeStats.append(mostNodes.elementAt(i));
		 			nodeStats.append(", ");
		 			nodeStats.append(leastNodes.elementAt(i));
		 			nodeStats.append(", ");
		 			nodeStats.append(meanNodes.elementAt(i));
		 			nodeStats.append(", ");
		 			nodeStats.append(medianNodes.elementAt(i));
		 			nodeStats.append(", ");
		 			nodeStats.append(nodesInBestCurrent.elementAt(i));
		 			nodeStats.append(", ");
		 			nodeStats.append(nodesInBestEver.elementAt(i));
		 			nodeStats.append("\r\n");
		 			outputFileNodeStats.write(nodeStats.toString());
	    		}
				outputFileNodeStats.close();
				
				//Dump the current generation fitness statistics
				fileName = "FitnessStats.txt";
				BufferedWriter outputFileFitnessStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
		 		outputFileFitnessStats.write("GenerationVSFitness, MaxFitness, MinFitness, MeanFitness, MedianFitness,  BestCurrent, BestEver \r\n");
		 		for(int i = 0; i < genFitnessMax.size(); i++)
	    		{
		 			StringBuffer fitnessStats = new StringBuffer();
		 			fitnessStats.append(i);
		 			fitnessStats.append(", ");
		 			fitnessStats.append(genFitnessMax.elementAt(i));
		 			fitnessStats.append(", ");
		 			fitnessStats.append(genFitnessMin.elementAt(i));
		 			fitnessStats.append(", ");
		 			fitnessStats.append(genFitnessMean.elementAt(i));
		 			fitnessStats.append(", ");
		 			fitnessStats.append(genFitnessMedian.elementAt(i));
		 			fitnessStats.append(", ");
		 			fitnessStats.append(genFitnessInBestCurrent.elementAt(i));
		 			fitnessStats.append(", ");
		 			fitnessStats.append(genFitnessInBestEver.elementAt(i));
		 			fitnessStats.append("\r\n");
		 			outputFileFitnessStats.write(fitnessStats.toString());
	    		}
				outputFileFitnessStats.close();
				
				//Dump the current generation collisions statistics
				fileName = "CollisionStats.txt";
				BufferedWriter outputFileCollisionsStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
		 		outputFileCollisionsStats.write("GenerationVSCollisions, MaxCollisions, MinCollisions, MeanCollisions, MedianCollisions,  BestCurrent, BestEver \r\n");
		 		for(int i = 0; i < genCollisionsMax.size(); i++)
	    		{
		 			StringBuffer collisionStats = new StringBuffer();
		 			collisionStats.append(i);
		 			collisionStats.append(", ");
		 			collisionStats.append(genCollisionsMax.elementAt(i));
		 			collisionStats.append(", ");
		 			collisionStats.append(genCollisionsMin.elementAt(i));
		 			collisionStats.append(", ");
		 			collisionStats.append(genCollisionsMean.elementAt(i));
		 			collisionStats.append(", ");
		 			collisionStats.append(genCollisionsMedian.elementAt(i));
		 			collisionStats.append(", ");
		 			collisionStats.append(genCollisionsBestCurrent.elementAt(i));
		 			collisionStats.append(", ");
		 			collisionStats.append(genCollisionsBestEver.elementAt(i));
		 			collisionStats.append("\r\n");
		 			outputFileCollisionsStats.write(collisionStats.toString());
	    		}
				outputFileCollisionsStats.close();
				
				//Dump the current generation DistanceToGoal statistics
				fileName = "DistanceToGoalStats.txt";
				BufferedWriter outputFileDistanceToGoalStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
		 		outputFileDistanceToGoalStats.write("GenerationVSDistanceToGoal, MaxDistanceToGoal, MinDistanceToGoal, MeanDistanceToGoal, MedianDistanceToGoal,  BestCurrent, BestEver \r\n");
		 		for(int i = 0; i < genDistanceToGoalMax.size(); i++)
	    		{
		 			StringBuffer distanceStats = new StringBuffer();
		 			distanceStats.append(i);
		 			distanceStats.append(", ");
		 			distanceStats.append(genDistanceToGoalMax.elementAt(i));
		 			distanceStats.append(", ");
		 			distanceStats.append(genDistanceToGoalMin.elementAt(i));
		 			distanceStats.append(", ");
		 			distanceStats.append(genDistanceToGoalMean.elementAt(i));
		 			distanceStats.append(", ");
		 			distanceStats.append(genDistanceToGoalMedian.elementAt(i));
		 			distanceStats.append(", ");
		 			distanceStats.append(genDistanceToGoalBestCurrent.elementAt(i));
		 			distanceStats.append(", ");
		 			distanceStats.append(genDistanceToGoalBestEver.elementAt(i));
		 			distanceStats.append("\r\n");
		 			outputFileDistanceToGoalStats.write(distanceStats.toString());
	    		}
				outputFileDistanceToGoalStats.close();
				
				//Dump the current generation NumEvaluations statistics
				fileName = "NumEvaluationsStats.txt";
				BufferedWriter outputFileNumEvaluationsStats = new BufferedWriter(new FileWriter(basePath+fileName, true));
		 		outputFileNumEvaluationsStats.write("GenerationVSNumEvaluations, MaxNumEvaluations, MinNumEvaluations, MeanNumEvaluations, MedianNumEvaluations,  BestCurrent, BestEver \r\n");
		 		for(int i = 0; i < genDistanceToGoalMax.size(); i++)
	    		{
		 			StringBuffer evaluationStats = new StringBuffer();
		 			evaluationStats.append(i);
		 			evaluationStats.append(", ");
		 			evaluationStats.append(genNumEvaluationsMax.elementAt(i));
		 			evaluationStats.append(", ");
		 			evaluationStats.append(genNumEvaluationsMin.elementAt(i));
		 			evaluationStats.append(", ");
		 			evaluationStats.append(genNumEvaluationsMean.elementAt(i));
		 			evaluationStats.append(", ");
		 			evaluationStats.append(genNumEvaluationsMedian.elementAt(i));
		 			evaluationStats.append(", ");
		 			evaluationStats.append(genNumEvaluationsBestCurrent.elementAt(i));
		 			evaluationStats.append(", ");
		 			evaluationStats.append(genNumEvaluationsBestEver.elementAt(i));
		 			evaluationStats.append("\r\n");
		 			outputFileNumEvaluationsStats.write(evaluationStats.toString());
	    		}
				outputFileNumEvaluationsStats.close();
				
				
				
	    	} catch (IOException e) {
	    		e.printStackTrace();
	    		Display.error(""+this.getClass().getName()+"::logGeneration(.) - cannot write logfile [" + basePath+fileName + "]");
	    	}	
	}
	
	public void logExperiment ( Population_NEATPopulation p)
	{
		String basePath = ((ParameterSet_Evolution_NEAT)p.getTemplate()).getExperimentPath();
		String fileName = "Experiment.ans";
		
		try
		{
			File oPath = new File(basePath);
			oPath.mkdirs();
			BufferedWriter outputFile = new BufferedWriter(new FileWriter(basePath+fileName, true));
			outputFile.write("Best individual ever is " + bestGenomeEver + "\r\n");
			outputFile.write("With the raw fitness: " + bestFitnessEver + "\r\n");
			outputFile.close();
		} catch( IOException e )
		{
    		e.printStackTrace();
    		Display.error(""+this.getClass().getName()+"::logExperiment(.) - cannot write logfile [" + basePath+fileName + "]");
		}
	}
	
	public void logXORExperiment ( Population_NEATPopulation p, String path)
	{
		String basePath = ((ParameterSet_Evolution_NEAT)p.getTemplate()).getExperimentPath();
		basePath += path + "/";
		String fileName = new String();
		
		//Set up test data
		//test data
		Vector<Double> input0 = new Vector<Double>(); //input values for node 0
		input0.add(0.0);
		input0.add(0.0);
		input0.add(1.0);
		input0.add(1.0);
		Vector<Double> input1 = new Vector<Double>(); //input values for node 1
		input1.add(0.0);
		input1.add(1.0);
		input1.add(0.0);
		input1.add(1.0);
		
		ArrayList<Double> inputValuesList = new ArrayList<Double>(); //input to the neural network
		
		RecurrentNeuralNetwork network = p.getRawBest().createPhenotype();
		//Dump the current species list to a file
		fileName = "BestGenomeXORTestOutput.txt";
		BufferedWriter outputFileSpeciesList;
		try {
			outputFileSpeciesList = new BufferedWriter(new FileWriter(basePath+fileName, true));
		
			for(int i = 0; i < input0.size(); i++)
			{
				inputValuesList.clear();
				inputValuesList.add(input0.get(i));
				inputValuesList.add(input1.get(i));
		
				network.step(inputValuesList);
		
				double output = network.getOutputNeuronAt(0).getOutputValue();

				outputFileSpeciesList.write("Input: " + input0.get(i) + " " + input1.get(i) + " Output: " + output + "\r\n");
			}
			outputFileSpeciesList.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void computeStatistics( Population_NEATPopulation p)
	{
		//Generations Champion
		Individual_NEATIndividual rawBest = p.getRawBest();
		
		//Current Champion Statistics
		cConnectionsInBestCurrent = rawBest.getNumLinks();
		cNodesInBestCurrent = rawBest.getNumNeurons();
		cGenFitnessMax = rawBest.getRawFitness();
		genFitnessInBestCurrent.add(rawBest.getRawFitness());
		for(int j = 0; j < gIDGenerationLog.size(); j++)
		{
			if(gIDGenerationLog.elementAt(j) == rawBest.genomeID)
			{
				genCollisionsBestCurrent.add(generationCollisionsLog.elementAt(j));
				genDistanceToGoalBestCurrent.add(generationDistanceLog.elementAt(j));
				genNumEvaluationsBestCurrent.add(generationEvaluationLog.elementAt(j));
			}
		}
		
		//Best Ever Champion Statistics
		if(rawBest.getRawFitness() > bestFitnessEver || genFitnessInBestEver.size() == 0)
		{
			//Champion Statistics
			bestFitnessEver = rawBest.getRawFitness();
			bestGenomeEver = "Generation" + p.getGenerationNumber() + "-> GenomeID:" + rawBest.genomeID;
			cConnectionsInBestEver = rawBest.getNumLinks();
			cNodesInBestEver = rawBest.getNumNeurons();
			genFitnessInBestEver.add(bestFitnessEver);
			for(int j = 0; j < gIDGenerationLog.size(); j++)
			{
				if(gIDGenerationLog.elementAt(j) == rawBest.genomeID)
				{
					genCollisionsBestEver.add(generationCollisionsLog.elementAt(j));
					genDistanceToGoalBestEver.add(generationDistanceLog.elementAt(j));
					genNumEvaluationsBestEver.add(generationEvaluationLog.elementAt(j));
				}
			}
		}
		else
		{
			cConnectionsInBestEver = connectionsInBestEver.elementAt(connectionsInBestEver.size()-1);
			cNodesInBestEver = nodesInBestEver.elementAt(nodesInBestEver.size()-1);
			genFitnessInBestEver.add(genFitnessInBestEver.elementAt(genFitnessInBestEver.size()-1));
			genCollisionsBestEver.add(genCollisionsBestEver.elementAt(genCollisionsBestEver.size()-1));
			genDistanceToGoalBestEver.add(genDistanceToGoalBestEver.elementAt(genDistanceToGoalBestEver.size()-1));
			genNumEvaluationsBestEver.add(genNumEvaluationsBestEver.elementAt(genNumEvaluationsBestEver.size()-1));
		}
		
		//set up the minimum variables so comparisons work.
		cLeastConnections = ((Individual_NEATIndividual)p.getIndividual(0)).getNumLinks();
		cLeastNodes = ((Individual_NEATIndividual)p.getIndividual(0)).getNumNeurons();
		cGenFitnessMin = ((Individual_NEATIndividual)p.getIndividual(0)).getRawFitness();
		for(int i = 0; i < p.populationSize; i++)
		{
			//Connection Statistics
			if(((Individual_NEATIndividual)p.getIndividual(i)).getNumLinks() > cMostConnections)
			{
				cMostConnections = ((Individual_NEATIndividual)p.getIndividual(i)).getNumLinks();
			}
			if(((Individual_NEATIndividual)p.getIndividual(i)).getNumLinks() < cLeastConnections)
			{
				cLeastConnections = ((Individual_NEATIndividual)p.getIndividual(i)).getNumLinks();
			}
			cMeanConnections += ((Individual_NEATIndividual)p.getIndividual(i)).getNumLinks();
			
			//Node Statistics
			if(((Individual_NEATIndividual)p.getIndividual(i)).getNumNeurons() > cMostNodes)
			{
				cMostNodes = ((Individual_NEATIndividual)p.getIndividual(i)).getNumNeurons();
			}
			if(((Individual_NEATIndividual)p.getIndividual(i)).getNumNeurons() < cLeastNodes)
			{
				cLeastNodes = ((Individual_NEATIndividual)p.getIndividual(i)).getNumNeurons();
			}
			cMeanNodes += ((Individual_NEATIndividual)p.getIndividual(i)).getNumNeurons();
			
			//Fitness Statistics
			if(((Individual_NEATIndividual)p.getIndividual(i)).getRawFitness() < cGenFitnessMin)
			{
				cGenFitnessMin = ((Individual_NEATIndividual)p.getIndividual(i)).getRawFitness();
			}
			cGenFitnessMean += ((Individual_NEATIndividual)p.getIndividual(i)).getRawFitness();
		}
		
		//Connection statistics	
		//Node statistics
		//Experiment statistics : Fitness
		if( p.populationSize % 2 == 0 )
		{
			cMedianConnections = ((Individual_NEATIndividual)p.getIndividual(p.populationSize/2)).getNumLinks();
			cMedianNodes = ((Individual_NEATIndividual)p.getIndividual(p.populationSize/2)).getNumNeurons();
			cGenFitnessMedian = ((Individual_NEATIndividual)p.getIndividual(p.populationSize/2)).getFitness();
		}
		else
		{
			cMedianConnections = ((Individual_NEATIndividual)p.getIndividual((p.populationSize/2)+1)).getNumLinks();
			cMedianNodes = ((Individual_NEATIndividual)p.getIndividual((p.populationSize/2)+1)).getNumNeurons();
			cGenFitnessMedian = ((Individual_NEATIndividual)p.getIndividual((p.populationSize/2)+1)).getFitness();
		}
		cMeanConnections = cMeanConnections/p.populationSize;
		cMeanNodes = cMeanNodes/p.populationSize;
		cGenFitnessMean = cGenFitnessMean/p.populationSize;
		
		//Connection Statistics - Add records for this generation
		mostConnections.add(cMostConnections);
		cMostConnections = 0;
		leastConnections.add(cLeastConnections);
		meanConnections.add(cMeanConnections);
		cMeanConnections = 0;
		medianConnections.add(cMedianConnections);
		cMedianConnections = 0;
		connectionsInBestEver.add(cConnectionsInBestEver);
		connectionsInBestCurrent.add(cConnectionsInBestCurrent);
		cConnectionsInBestCurrent = 0;
		
		//Node statistics - Add records for this generation
		mostNodes.add(cMostNodes);
		cMostNodes = 0;
		leastNodes.add(cLeastNodes);
		meanNodes.add(cMeanNodes);
		cMeanNodes = 0;
		medianNodes.add(cMedianNodes);
		cMedianNodes = 0;
		nodesInBestEver.add(cNodesInBestEver);
		nodesInBestCurrent.add(cNodesInBestCurrent);
		cNodesInBestCurrent  = 0;
		
		//Experiment statistics - Add records for this generation
		genFitnessMax.add(cGenFitnessMax);
		cGenFitnessMax = 0;
		genFitnessMin.add(cGenFitnessMin);
		genFitnessMean.add(cGenFitnessMean);
		cGenFitnessMean = 0;
		genFitnessMedian.add(cGenFitnessMedian);
		cGenFitnessMedian = 0;
		
		//Species Statistics
		Vector<Integer> speciesRecord = new Vector<Integer>();
		for(int i = 0; i < p.speciesList.maxSpeciesNumber(); i++)
		{
			speciesRecord.add(0);
		}
		for(int i = 0; i < p.speciesList.numOfSpecies(); i++)
		{
			speciesRecord.set(p.speciesList.getSpeciesAt(i).speciesID, p.speciesList.getSpeciesAt(i).size());
		}
		speciesGenerationsVsSize.add(speciesRecord);
		speciesRecord = null;
		
		//Innovation Statistics
		numberOfInnovations.add(p.innovationList.size());
		if(numberOfInnovations.size() < 2)
			numberOfNewInnovations.add(p.innovationList.size() - numberOfInnovations.elementAt(numberOfInnovations.size()-1));
		else
			numberOfNewInnovations.add(numberOfInnovations.elementAt(numberOfInnovations.size()-1) - numberOfInnovations.elementAt(numberOfInnovations.size()-2));
		calculateGenerationStats();
	}
	
	/** Parameters: GenomeID, Number of Collisions, Distance from Goal, Number of Evaluations */
	public void logCollisionsDistanceEvals(int gID, int col, double dis, int evals)
	{
		gIDGenerationLog.add(gID);
		generationCollisionsLog.add(col);
		generationDistanceLog.add(dis);
		generationEvaluationLog.add(evals);
	}
	
	public void resetGenerationStats()
	{
		gIDGenerationLog.clear();
		generationCollisionsLog.clear();
		generationDistanceLog.clear();
		generationEvaluationLog.clear();
	}
	
	public  void calculateGenerationStats()
	{
		int generationSize = gIDGenerationLog.size();
		
		if(generationSize == 0)
		{
			Display.error(""+this.getClass().getName()+"::calculateGenerationStats(.) - log data not recorded, make certain you are calling logCollisionsDistanceEvals");
		}
		else if(generationCollisionsLog.size() != generationSize ||
		   generationDistanceLog.size() != generationSize ||
		   generationEvaluationLog.size() != generationSize)
		{
			Display.error(""+this.getClass().getName()+"::calculateGenerationStats(.) - log size mismatch, something is not logging correctly");
		}
		else
		{
			int highCol = 0;
			int lowCol = generationCollisionsLog.elementAt(0);
			double meanCol = 0;
			double highDist = 0;
			double lowDist = generationDistanceLog.elementAt(0);
			double meanDist = 0;
			int highEvals = 0;
			int lowEvals = generationEvaluationLog.elementAt(0);
			double meanEvals = 0;
			for(int i = 0; i < generationSize; i++)
			{
				//Log Collisions
				if(generationCollisionsLog.elementAt(i) > highCol)
				{
					highCol = generationCollisionsLog.elementAt(i);
				}
				if(generationCollisionsLog.elementAt(i) < lowCol)
				{
					lowCol = generationCollisionsLog.elementAt(i);
				}
				meanCol += generationCollisionsLog.elementAt(i);
				
				//Log Distances
				if(generationDistanceLog.elementAt(i) > highDist)
				{
					highDist = generationDistanceLog.elementAt(i);
				}
				if(generationDistanceLog.elementAt(i) < lowDist)
				{
					lowDist = generationDistanceLog.elementAt(i);
				}
				meanDist += generationDistanceLog.elementAt(i);
				
				//Log Evaluations
				if(generationEvaluationLog.elementAt(i) > highEvals)
				{
					highEvals = generationEvaluationLog.elementAt(i);
				}
				if(generationEvaluationLog.elementAt(i) < lowEvals)
				{
					lowEvals = generationEvaluationLog.elementAt(i);
				}
				meanEvals += generationEvaluationLog.elementAt(i);
			}
			
			genCollisionsMax.add(highCol);
			genCollisionsMin.add(lowCol);
			genDistanceToGoalMax.add(highDist);
			genDistanceToGoalMin.add(lowDist);
			genNumEvaluationsMax.add(highEvals);
			genNumEvaluationsMin.add(lowEvals);
			
			if( gIDGenerationLog.size() % 2 == 0 )
			{
				genCollisionsMedian.add(generationCollisionsLog.elementAt(generationCollisionsLog.size()/2));
				genDistanceToGoalMedian.add(generationDistanceLog.elementAt(generationDistanceLog.size()/2));
				genNumEvaluationsMedian.add(generationEvaluationLog.elementAt(generationEvaluationLog.size()/2));
			}
			else
			{
				genCollisionsMedian.add(generationCollisionsLog.elementAt((generationCollisionsLog.size()/2)+1));
				genDistanceToGoalMedian.add(generationDistanceLog.elementAt((generationDistanceLog.size()/2)+1));
				genNumEvaluationsMedian.add(generationEvaluationLog.elementAt((generationEvaluationLog.size()/2)+1));
			}
			genCollisionsMean.add(meanCol/generationCollisionsLog.size());
			genDistanceToGoalMean.add(meanDist/generationDistanceLog.size());
			genNumEvaluationsMean.add(meanEvals/generationEvaluationLog.size());
		}
		resetGenerationStats();
	}
}
