/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.evolution;

import java.util.*;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.core.representation.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.*;

public class VariationOperator_Individual_NEAT_MutationAddLink extends VariationOperator {

	//rate from VariationOperator will correspond to mutationRate
	protected double mutationRate;
	protected double chanceOfLoopedLink;
	protected Population_NEATPopulation_InnovationList innovationList;
	protected int numTrysToFindLoop;
	protected int numTrysToAddLink;
	//create a random number generator
	Random rand;
	
	public VariationOperator_Individual_NEAT_MutationAddLink(String __name) {
		super(__name, 0.0);
		mutationRate = 0.0;
		chanceOfLoopedLink = 0.0;
		innovationList = null;
		numTrysToFindLoop = 1;
		numTrysToAddLink = 1;
		rand = new Random();
	}

	public VariationOperator_Individual_NEAT_MutationAddLink(String __name, double rate) {
		super(__name, rate);
		mutationRate = rate;
		chanceOfLoopedLink = 0.0;
		innovationList = null;
		numTrysToFindLoop = 1;
		numTrysToAddLink = 1;
		rand = new Random();
	}

	public VariationOperator_Individual_NEAT_MutationAddLink(String __name, double _chanceOfLoopedLink,
											Population_NEATPopulation_InnovationList _innovationList,
											int _numTrysToFindLoop, int _numTrysToAddLink) {
		super(__name, 0.0);
		mutationRate = 0.0;
		chanceOfLoopedLink = _chanceOfLoopedLink;
		innovationList = _innovationList;
		numTrysToFindLoop = _numTrysToFindLoop;
		numTrysToAddLink = _numTrysToAddLink;
		rand = new Random();
	}

	public VariationOperator_Individual_NEAT_MutationAddLink(String __name, double rate,
											double _chanceOfLoopedLink,
											Population_NEATPopulation_InnovationList _innovationList,
											int _numTrysToFindLoop, int _numTrysToAddLink) {
		super(__name, rate);
		mutationRate = rate;
		chanceOfLoopedLink = _chanceOfLoopedLink;
		innovationList = _innovationList;
		numTrysToFindLoop = _numTrysToFindLoop;
		numTrysToAddLink = _numTrysToAddLink;
		rand = new Random();
	}
	
	@Override
	public void applyOperator(EvolutionObject object) {

		Individual_NEATIndividual genome = (Individual_NEATIndividual)object;
		int defaultNeuronInputType = 1;
		int defaultNeuronOutputType = 2;
		int defaultNeuronHiddenType = 3;
		int defaultNeuronBiasType = 4; //0=none, 1=input, 2=output, 3=hidden, 4=bias
		
		if(Math.random() > mutationRate)
		{
			return;
		}
		
		//define holders for the two neurons to be linked. If we have found two 
		//valid neurons to link these values will become >= 0.
		int neuronToBeLinked1 = -1;
		int neuronToBeLinked2 = -1;

		int neuronToCheck = -1;
		int rand1 = -1;
		int rand2 = -1;
		
		//flag set if a recurrent link is selected
		boolean isRecurrent = false;
		
		if(Math.random() < chanceOfLoopedLink)
		{
			//If yes, then try numTrysToFindLoop to find a neuron that is not an
			//input or bias neuron and that does not already have a loopback connection
			//IMPORTANT if you add the bias node in InitializationOperator you must add +1 to genome.inputs below
			while(numTrysToFindLoop-- > 0)
			{
				//grab a random neuron
				neuronToCheck = randInt(genome.inputs,genome.getNumNeurons()-1);
				
				//check to make sure the neuron does not already have a loopback link
				//and that it is not an input or bias neuron
				if(!genome.getNeuronAt(neuronToCheck).getRecurrent() &&
				  (genome.getNeuronAt(neuronToCheck).getNeuronType() != defaultNeuronBiasType) &&
				  (genome.getNeuronAt(neuronToCheck).getNeuronType() != defaultNeuronInputType))
				{
					neuronToBeLinked1 = genome.getNeuronAt(neuronToCheck).getNeuronID();
					neuronToBeLinked2 = neuronToBeLinked1;
					isRecurrent = true;
					numTrysToFindLoop = 0;
				}
				if( neuronToBeLinked2 == 0 || neuronToBeLinked2 == 1)
				{
					Display.info("About to create an invalid link");
				}
			}
		}
		else
		{
			//If no, try to find two unlinked neurons. Make numTrysToAddLink attempts
			//IMPORTANT if you add the bias node in InitializationOperator you must add +1 to genome.inputs below
			while(numTrysToAddLink-- > 0)
			{
				rand1 = randInt(0,genome.getNumNeurons()-1);
				rand2 = randInt(genome.inputs,genome.getNumNeurons()-1);
				
				Element_NEAT_NGene neuron1 = genome.getNeuronAt(rand1);
				Element_NEAT_NGene neuron2 = genome.getNeuronAt(rand2);
				
				neuronToBeLinked1 = neuron1.getNeuronID();
				neuronToBeLinked2 = neuron2.getNeuronID();
				
				if(genome.getNeuronAt(rand2).getNeuronType() == defaultNeuronInputType)
				{
					continue;
				}
				if(duplicateLink(neuronToBeLinked1, neuronToBeLinked2, genome) ||
				   neuronToBeLinked1 == neuronToBeLinked2 ||
				   (neuron1.getNeuronType() == defaultNeuronOutputType && neuron2.getNeuronType() == defaultNeuronOutputType) )
				{
					neuronToBeLinked1 = -1;
					neuronToBeLinked2 = -1;
				}
				else
				{
					numTrysToAddLink = 0;
				}
				if( neuronToBeLinked2 == 0 || neuronToBeLinked2 == 1)
				{
					Display.info("About to create an invalid link");
				}
			}
		}
		
		//return if unsuccessful in finding a link
		if( neuronToBeLinked1 < 0 || neuronToBeLinked2 < 0)
		{
			return;
		}
		
		//check to see if we have already created this innovation
		//since we are adding a link, the innovation type is 1
		int id = innovationList.checkInnovation(neuronToBeLinked1, neuronToBeLinked2, 1);
		
		//is this link recurrent?
		if(genome.getNeuronAt(getElementPosition(neuronToBeLinked1,genome)).getSplitY() >
		   genome.getNeuronAt(getElementPosition(neuronToBeLinked2,genome)).getSplitY())
		{
			isRecurrent = true;
		}
		
		if(id < 0)
		{
			//get the next innovation id
			int idLink1 = innovationList.getNextInnovationID();
			//create a new innovation
			innovationList.createNewInnovation(neuronToBeLinked1, neuronToBeLinked2, 1);
			
			//create a new gene
			VariationOperator_Element_NEAT_Gene op = new VariationOperator_Element_NEAT_Gene("initialization operator",
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getProbabilityRateReplaced(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getProbabilityRateReplaced(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getMaxWeightPerturbation(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getActivationMutationRate(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getMaxActivationPerturbation());
			if( neuronToBeLinked2 == 0 || neuronToBeLinked2 == 1)
			{
				Display.info("About to create an invalid link");
			}
			
			Element_NEAT_LGene nLink = new Element_NEAT_LGene("mutation link - new inno",
															  genome,
															  op,
															  idLink1,
															  isRecurrent,
															  neuronToBeLinked1,
															  neuronToBeLinked2,
															  1.0);
			genome.addLink(nLink);
		}
		else
		{
			//This innovation has already been created so all we need to do is create
			//the new gene using the existing innovation ID
			VariationOperator_Element_NEAT_Gene op = new VariationOperator_Element_NEAT_Gene("initialization operator",
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getProbabilityRateReplaced(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getProbabilityRateReplaced(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getMaxWeightPerturbation(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getActivationMutationRate(),
																							((ParameterSet_Evolution_NEAT)genome.getTemplate()).getMaxActivationPerturbation());
			Element_NEAT_LGene nLink = new Element_NEAT_LGene("mutation link - old inno",
															  genome,
															  op,
															  id,
															  isRecurrent,
															  neuronToBeLinked1,
															  neuronToBeLinked2,
															  1.0);
			genome.addLink(nLink);
		}
		return;
	}
	
	//given a neuron ID this function finds its position in the vector of neurons
	private int getElementPosition(int neuronId, Individual_NEATIndividual genome)
	{
		for(int neuron = 0; neuron < genome.getNumNeurons(); neuron++)
		{
			if(genome.getNeuronAt(neuron).getNeuronID() == neuronId)
			{
				return neuron;
			}
		}
		Display.error(""+this.getClass().getName()+"::mutate(.) - Attempting to a link to a node which apparently doesn't exist in this genome");
		return -1;
	}
	
	//returns true if the given link is already part of the given genome
	//returns false if it is new.
	private boolean duplicateLink(int neuronIn, int neuronOut, Individual_NEATIndividual genome)
	{
		for(int link = 0; link < genome.getNumLinks(); link++)
		{
			if(genome.getLinkAt(link).getFromNeuron() == neuronIn &&
			   genome.getLinkAt(link).getToNeuron() == neuronOut)
			{
				return true;
			}
		}
		return false;
	}
	
	//returns a random integer between x and y
	private int randInt(int x, int y)
	{
		return rand.nextInt(Integer.MAX_VALUE)%(y-x+1)+x;
	}
}
