/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.representation;

import java.util.ArrayList;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.core.representation.*;
import simbad.picoevo.toolbox.*;

public class Element_NEAT_LGene extends Element_NEAT_Gene implements java.io.Serializable{

	protected int fromNeuron;
	protected int toNeuron;
	protected double weight;
	
	// *** Constructors ***
	public Element_NEAT_LGene() {
		super("empty", null, null);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}
	
	public Element_NEAT_LGene(String __name, Individual owner, VariationOperator levelOperator) {
		super(__name, owner, levelOperator);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}

	public Element_NEAT_LGene(String __name, Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent) {
		super(__name, owner, levelOperator, _innovationid, _recurrent);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}

	public Element_NEAT_LGene(String __name, Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent) {
		super(__name, owner, levelOperatorList, _innovationid, _recurrent);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}

	public Element_NEAT_LGene(String __name, Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent,
								int _fromNeuron, int _toNeuron, double _weight) {
		super(__name, owner, levelOperator, _innovationid, _recurrent);
		fromNeuron = _fromNeuron;
		toNeuron = _toNeuron;
		weight = _weight;
	}

	public Element_NEAT_LGene(String __name, Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent,
								int _fromNeuron, int _toNeuron, double _weight) {
		super(__name, owner, levelOperatorList, _innovationid, _recurrent);
		fromNeuron = _fromNeuron;
		toNeuron = _toNeuron;
		weight = _weight;
	}
	
	public Element_NEAT_LGene(Individual owner, VariationOperator levelOperator) {
		super(owner, levelOperator);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}

	public Element_NEAT_LGene(Individual owner, ArrayList levelOperatorList) {
		super(owner, levelOperatorList);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}

	public Element_NEAT_LGene(Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent) {
		super(owner, levelOperator, _innovationid, _recurrent);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}

	public Element_NEAT_LGene(Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent) {
		super(owner, levelOperatorList, _innovationid, _recurrent);
		fromNeuron = -1;
		toNeuron = -1;
		weight = 0.0;
	}

	public Element_NEAT_LGene(Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent,
								int _fromNeuron, int _toNeuron, double _weight) {
		super(owner, levelOperator, _innovationid, _recurrent);
		fromNeuron = _fromNeuron;
		toNeuron = _toNeuron;
		weight = _weight;
	}

	public Element_NEAT_LGene(Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent,
								int _fromNeuron, int _toNeuron, double _weight) {
		super(owner, levelOperatorList, _innovationid, _recurrent);
		fromNeuron = _fromNeuron;
		toNeuron = _toNeuron;
		weight = _weight;
	}
	
	// *** End Constructors ***
	
	public int getFromNeuron()
	{
		return fromNeuron;
	}
	
	public void setFromNeuron(int n)
	{
		fromNeuron = n;
	}
	
	public int getToNeuron()
	{
		return toNeuron;
	}
	
	public void setToNeuron(int n)
	{
		toNeuron = n;
	}
	
	public double getLinkWeight()
	{
		return weight;
	}
	
	public void setLinkWeight(double w)
	{
		weight = w;
	}
	public void displayInformation()
	{
		super.displayInformation();
		Display.info_nocr(" ( \""+_name+"\",{");
		Display.info_nocr(" fromNeuron: "+fromNeuron);
		Display.info_nocr(" toNeuron: "+toNeuron);
		Display.info_nocr(" weight: "+weight);
		Display.info_nocr(" )");
	}
	
	public String toString()
	{
		StringBuffer output = new StringBuffer();
		output.append(super.toString());
		output.append("L: {");
		output.append(fromNeuron);
		output.append("->(");
		output.append(weight);
		output.append(")->");
		output.append(toNeuron);
		output.append("}");
		return output.toString();
	}
	
	public Object clone()
	{
		Element_NEAT_LGene g = null;
		
		try 
		{
			g = (Element_NEAT_LGene)super.clone();
		}
		catch (InternalError e) 
		{
            // This should never happen
            throw new InternalError(e.toString());
		}
		g.fromNeuron = fromNeuron;
		g.toNeuron = toNeuron;
		g.weight = weight;
		
		return g;
	}
	
	public int compareTo(Object o) {
		return this.innovationID - ((Element_NEAT_Gene)o).innovationID;
	}
}
