/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.representation;

import java.util.ArrayList;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.core.representation.*;
import simbad.picoevo.toolbox.*;

public class Element_NEAT_NGene extends Element_NEAT_Gene implements java.io.Serializable{

	protected int neuronType; //0=none, 1=input, 2=output, 3=hidden, 4=bias
	protected double activationResponse;
	protected int neuronID; //unique to Individual id for this neuron.
	double splitX;
	double splitY;
	
	// *** Constructors ***
	
	public Element_NEAT_NGene() {
		super("empty", null , null);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}
	
	public Element_NEAT_NGene(String __name, Individual owner, VariationOperator levelOperator) {
		super(__name, owner, levelOperator);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}

	public Element_NEAT_NGene(String __name, Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent) {
		super(__name, owner, levelOperator, _innovationid, _recurrent);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}

	public Element_NEAT_NGene(String __name, Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent) {
		super(__name, owner, levelOperatorList, _innovationid, _recurrent);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}

	public Element_NEAT_NGene(String __name, Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent,
								int _neuronType, double _activationResponse, double _splitX, double _splitY, int _neuronID) {
		super(__name, owner, levelOperator, _innovationid, _recurrent);
		neuronType = _neuronType;
		activationResponse = _activationResponse;
		neuronID = _neuronID;
		splitX = _splitX;
		splitY = _splitY;
	}

	public Element_NEAT_NGene(String __name, Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent,
								int _neuronType, double _activationResponse, double _splitX, double _splitY, int _neuronID) {
		super(__name, owner, levelOperatorList, _innovationid, _recurrent);
		neuronType = _neuronType;
		activationResponse = _activationResponse;
		neuronID = _neuronID;
		splitX = _splitX;
		splitY = _splitY;
	}
	
	public Element_NEAT_NGene(Individual owner, VariationOperator levelOperator) {
		super(owner, levelOperator);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}

	public Element_NEAT_NGene(Individual owner, ArrayList levelOperatorList) {
		super(owner, levelOperatorList);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}

	public Element_NEAT_NGene(Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent) {
		super(owner, levelOperator, _innovationid, _recurrent);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}

	public Element_NEAT_NGene(Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent) {
		super(owner, levelOperatorList, _innovationid, _recurrent);
		neuronType = 0;
		activationResponse = 0.0;
		neuronID = 0;
		splitX = 0.0;
		splitY = 0.0;
	}

	public Element_NEAT_NGene(Individual owner, VariationOperator levelOperator, int _innovationid, boolean _recurrent,
								int _neuronType, double _activationResponse, double _splitX, double _splitY, int _neuronID) {
		super(owner, levelOperator, _innovationid, _recurrent);
		neuronType = _neuronType;
		activationResponse = _activationResponse;
		neuronID = _neuronID;
		splitX = _splitX;
		splitY = _splitY;
	}

	public Element_NEAT_NGene(Individual owner, ArrayList levelOperatorList, int _innovationid, boolean _recurrent,
								int _neuronType, double _activationResponse, double _splitX, double _splitY, int _neuronID) {
		super(owner, levelOperatorList, _innovationid, _recurrent);
		neuronType = _neuronType;
		activationResponse = _activationResponse;
		neuronID = _neuronID;
		splitX = _splitX;
		splitY = _splitY;
	}
	
	public void setNeuronType(int i)
	{
		if ( i > 4 || i < 0 )
			Display.error(""+this.getClass().getName()+"::setNeuronType - invalid neuron type");
		else
			neuronType = i;
	}
	
	public int getNeuronType()
	{
		return neuronType;
	}
	
	public void setActivationResponse(double a)
	{
		activationResponse = a;
	}
	
	public double getActivationResponse()
	{
		return activationResponse;
	}
	
	public int getNeuronID() {
		return neuronID;
	}

	public void setNeuronID(int neuronID) {
		this.neuronID = neuronID;
	}

	public double getSplitX() {
		return splitX;
	}

	public void setSplitX(double splitX) {
		this.splitX = splitX;
	}

	public double getSplitY() {
		return splitY;
	}

	public void setSplitY(double splitY) {
		this.splitY = splitY;
	}

	public void displayInformation()
	{
		super.displayInformation();
		Display.info_nocr(" ( \""+_name+"\",{");
		Display.info_nocr(" neuronType: "+neuronType);
		Display.info_nocr(" activationResponse: "+activationResponse);
		Display.info_nocr(" neuronID: "+neuronID);
		Display.info_nocr(" splitX: "+splitX);
		Display.info_nocr(" splitY: "+splitY);
		Display.info_nocr(" )");
	}
	
	public String toString()
	{
		//0=none, 1=input, 2=output, 3=hidden, 4=bias
		String type = new String();
		StringBuffer output = new StringBuffer();
		
		if(neuronType == 0)
			type = "none";
		else if(neuronType == 1)
			type = "input";
		else if(neuronType == 2)
			type = "output";
		else if(neuronType == 3)
			type = "hidden";
		else if(neuronType == 4)
			type = "bias";
		
		output.append(super.toString());
		output.append("N: nID: ");
		output.append(neuronID);
		output.append(" = {");
		output.append(type);
		output.append(": (");
		output.append(activationResponse);
		output.append(")}");
		return output.toString();
	}
	
	public Object clone()
	{
		Element_NEAT_NGene g = null;
		
		try 
		{
			g = (Element_NEAT_NGene)super.clone();
		}
		catch (InternalError e) 
		{
            // This should never happen
            throw new InternalError(e.toString());
		}
		g.neuronType = neuronType;
		g.activationResponse = activationResponse;
		g.splitX = splitX;
		g.splitY = splitY;
		
		return g;
	}
	
	public int compareTo(Object o) {
		return this.innovationID - ((Element_NEAT_Gene)o).innovationID;
	}
}
