/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.representation;

import java.util.ArrayList;
import java.util.Vector;
import java.util.Collections;
import java.util.HashMap;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.core.representation.*;
import simbad.picoevo.ext.evolution.*;
import simbad.picoevo.toolbox.*;

import simbad.piconode.core.node.*;
import simbad.piconode.core.arc.*;
import simbad.piconode.ext.*;

public class Individual_NEATIndividual extends Individual implements Comparable{

	public int genomeID;	//individual identification number
	protected Vector<Element_NEAT_NGene> genomeNeurons; //the neurons that make up this genome
	//This hashmap contains one entry for every unique Innovation ID in the genomes Node List that are
	//enabled. The Innovation ID acts as the key value for a map to the appropriate neuron in the phenotypes
	//neuron list.
	protected HashMap<Integer,Integer> iIDToNeuron = new HashMap<Integer,Integer>();
	protected Vector<Element_NEAT_LGene> genomeLinks;	//the links that connect the neurons in this genome
	//This hashmap contains one entry for every unique Innovation ID in the genomes Link List that are
	//enabled. The Innovation ID acts as the key value for a map to the appropriate link in the phenotypes
	//link list.
	protected HashMap<Integer,Integer> iIDToLink = new HashMap<Integer,Integer>();
	protected int genomeNetDepth; //the depth of this genomes network
    protected double genomeRawFitness; //the raw fitness score of this genome (this will be calculated by the simulator)
    protected double genomeAdjustedFitness; //the fitness score after placed in a species and adjusted accordingly
    protected double amountToSpawn; //the amount of children this genome is to spawn
    public int inputs, outputs; //basic record of number of inputs and outputs
    public int genomeSpecies; //track species genome belongs to, purely for record keeping
    
	public Individual_NEATIndividual(String __name, Population owner) {
		super(__name, owner);
		genomeID = 0;
		genomeNeurons = new Vector<Element_NEAT_NGene>();
		genomeLinks = new Vector<Element_NEAT_LGene>();
		genomeNetDepth = 0;
		genomeRawFitness = -1;
		genomeAdjustedFitness = 0;
		amountToSpawn = 0;
		inputs = 0;
		outputs = 0;
		genomeSpecies = 0;
	}

	public Individual_NEATIndividual(Population owner) {
		super(owner);
		genomeID = 0;
		genomeNeurons = new Vector<Element_NEAT_NGene>();
		genomeLinks = new Vector<Element_NEAT_LGene>();
		genomeNetDepth = 0;
		genomeRawFitness = -1;
		genomeAdjustedFitness = 0;
		amountToSpawn = 0;
		inputs = 0;
		outputs = 0;
		genomeSpecies = 0;
	}
	public Individual_NEATIndividual(String __name, Population owner, int GID,
			int depth, int ins, int outs, int species) {
		super(__name, owner);
		genomeID = GID;
		genomeNeurons = new Vector<Element_NEAT_NGene>();
		genomeLinks = new Vector<Element_NEAT_LGene>();
		genomeNetDepth = depth;
		genomeRawFitness = -1;
		genomeAdjustedFitness = 0;
		amountToSpawn = 0;
		inputs = ins;
		outputs = outs;
		genomeSpecies = species;
	}
	
	public Individual_NEATIndividual(String __name, Population owner, int GID, 
									Vector<Element_NEAT_NGene> nodes, Vector<Element_NEAT_LGene> links,
									int depth, int ins, int outs, int species) {
		super(__name, owner);
		genomeID = GID;
		genomeNeurons = new Vector<Element_NEAT_NGene>();
		genomeLinks = new Vector<Element_NEAT_LGene>();
		for(int i = 0; i < nodes.size(); i++)
			genomeNeurons.add((Element_NEAT_NGene)nodes.get(i).clone());
		for(int i = 0; i < links.size(); i++)
			genomeLinks.add((Element_NEAT_LGene)links.get(i).clone());
		correctOwnership();
		rebuildIIDLookup();
		genomeNetDepth = depth;
		genomeRawFitness = -1;
		genomeAdjustedFitness = 0;
		amountToSpawn = 0;
		inputs = ins;
		outputs = outs;
		genomeSpecies = species;
		
	}

	public Individual_NEATIndividual(Population owner, int GID, 
									Vector<Element_NEAT_NGene> nodes, Vector<Element_NEAT_LGene> links, 
									int depth, int ins, int outs, int species) {
		super(owner);
		genomeID = GID;
		genomeNeurons = new Vector<Element_NEAT_NGene>();
		genomeLinks = new Vector<Element_NEAT_LGene>();
		for(int i = 0; i < nodes.size(); i++)
			genomeNeurons.add((Element_NEAT_NGene)nodes.get(i).clone());
		for(int i = 0; i < links.size(); i++)
			genomeLinks.add((Element_NEAT_LGene)links.get(i).clone());
		correctOwnership();
		rebuildIIDLookup();
		genomeNetDepth = depth;
		genomeRawFitness = -1;
		genomeAdjustedFitness = 0;
		amountToSpawn = 0;
		inputs = ins;
		outputs = outs;
		genomeSpecies = species;
	}
	
	private void correctOwnership()
	{
		for(int i = 0; i < genomeNeurons.size(); i++)
			genomeNeurons.elementAt(i).setOwner(this);
		for(int i = 0; i < genomeLinks.size(); i++)
			genomeLinks.elementAt(i).setOwner(this);
	}
	
	public void rebuildIIDLookup()
	{
		iIDToNeuron.clear();
		iIDToLink.clear();
		for(int i = 0; i < genomeNeurons.size(); i++)
			iIDToNeuron.put(((Element_NEAT_Gene)genomeNeurons.get(i)).getInnovationID(), i);
		for(int i = 0; i < genomeLinks.size(); i++)
			iIDToLink.put(((Element_NEAT_Gene)genomeLinks.get(i)).getInnovationID(), i);
	}
	
	public void displayInformation()
	{
		Display.info(" ( \""+_name+"\",{");
		Display.info(" genomeID :"+genomeID);
		for(int i = 0; i < genomeNeurons.size(); i++)
		{
			Display.info_nocr(""+i+":");
			genomeNeurons.get(i).displayInformation();
			Display.info("end "+i);
		}
		for(int i = 0; i < genomeLinks.size(); i++)
		{
			Display.info_nocr(""+i+":");
			genomeLinks.get(i).displayInformation();
			Display.info("end "+i);
		}
		Display.info(" genomeNetDepth :"+genomeNetDepth);
		Display.info(" genomeRawFitness :"+genomeRawFitness);
		Display.info(" genomeAdjustedFitness :"+genomeAdjustedFitness);
		Display.info(" amountToSpawn :"+amountToSpawn);
		Display.info(" num inputs :"+inputs);
		Display.info(" num outputs :"+outputs);
		Display.info(" genomeSpecies :"+genomeSpecies);
		Display.info(" }");
	}
	
	public String toString()
	{
		StringBuffer output = new StringBuffer();
		output.append(_name);
		output.append(" / gID: ");
		output.append(genomeID);
		output.append("\r\n");
		output.append("    RawFit:");
		output.append(genomeRawFitness);
		output.append(" AdjFit:");
		output.append(genomeAdjustedFitness);
		output.append(" AmtSpwn:");
		output.append(amountToSpawn);
		output.append(" Species:");
		output.append(genomeSpecies);
		output.append("\r\n{\r\n");
		for(int i = 0; i < genomeNeurons.size(); i++)
		{
			output.append(genomeNeurons.get(i).toString());
			output.append("\r\n");
		}
		for(int i = 0; i < genomeLinks.size(); i++)
		{
			output.append(genomeLinks.get(i).toString());
			output.append("\r\n");
		}
		output.append("}\r\n");
		return output.toString();
 	}
	
	public void addNeuron(Element_NEAT_NGene n)
	{
		genomeNeurons.add(n);
		rebuildIIDLookup();
	}
	
	public Element_NEAT_NGene getNeuronAt(int i)
	{
		if(i < genomeNeurons.size())
			return genomeNeurons.get(i);
		else
		{
			Display.error(""+this.getClass().getName()+"::getNeuronAt - neuron at " + i + " does not exist");
			return null;
		}
	}
	
	public void addLink(Element_NEAT_LGene n)
	{
		genomeLinks.add(n);
		rebuildIIDLookup();
	}
	
	public Element_NEAT_LGene getLinkAt(int i)
	{
		if(i < genomeLinks.size())
			return genomeLinks.get(i);
		else
		{
			Display.error(""+this.getClass().getName()+"::getLinkAt - link at " + i + " does not exist");
			return null;
		}
	}
	
	public void setNetDepth(int depth)
	{
		genomeNetDepth = depth;
	}
	
	public int getNetDepth()
	{
		return genomeNetDepth;
	}
	
	public void setRawFitness(double fit)
	{
		_fitness = fit;
		genomeRawFitness = fit;
	}
	
	public double getRawFitness()
	{
		return genomeRawFitness;
	}
	
	public void setAdjustedFitness(double fit)
	{
		setEvaluationFlag(true);
		_fitness = fit;
		genomeAdjustedFitness = fit;
	}
	
	public double getAdjustedFitness()
	{
		return genomeAdjustedFitness;
	}
	
	public void setAmountToSpawn(double fit)
	{
		amountToSpawn = fit;
	}
	
	public double getAmountToSpawn()
	{
		return amountToSpawn;
	}
	
	public void setInputsOutputs(int in, int out)
	{
		inputs = in;
		outputs = out;
	}
	
	public int numInputs()
	{
		return inputs;
	}
	
	public int numOutputs()
	{
		return outputs;
	}
	
	public Vector<Element_NEAT_LGene> getLinks()
	{
		return genomeLinks;
	}
	
	public void setAllLinks(Vector<Element_NEAT_LGene> links)
	{
		genomeLinks.clear();
		for(int i = 0; i < links.size(); i++)
			genomeLinks.add((Element_NEAT_LGene)links.get(i).clone());
		rebuildIIDLookup();
	}
	
	public Vector<Element_NEAT_NGene> getNeurons()
	{
		return genomeNeurons;
	}
	
	public void setAllNeurons(Vector<Element_NEAT_NGene> nodes)
	{
		genomeNeurons.clear();
		for(int i = 0; i < nodes.size(); i++)
			genomeNeurons.add((Element_NEAT_NGene)nodes.get(i).clone());
		rebuildIIDLookup();
	}
	
	public int getNumNeurons()
	{
		return genomeNeurons.size();
	}
	
	public int getNumLinks()
	{
		return genomeLinks.size();
	}
	
	public int getGenomeID() {
		return genomeID;
	}

	public void setGenomeID(int genomeID) {
		this.genomeID = genomeID;
	}

	/** Calculate the distance between two genomes according to the formula
	 * 
	 * CompatDist = ((c1*#ExcessGenes)/N) + ((c2*#DisjointGenes)/N) + c3*WeightDifference
	 * 
	 * N = number of genes in the larger genome
	 * c1,c2,c3 = coefficients to tweak outcome
	 *
	 * Original Algorithm written by Mat Buckland http://www.ai-junkie.com
	 * Ported to this system by Robert Lucas http://www.automatonsadrift.com
	 *
	 * @param genome
	 * @return double
	 */
	public double getCompatibilityScore(Individual_NEATIndividual genome)
	{
		//Travel down the length of each genome and count the number of disjoint
		//genes, excess genes and matched genes
		double numDisjoint = 0;
		double numExcess = 0;
		double numMatched = 0;
		
		//records the summed difference of weights in matched genes
		double weightDifference = 0;
		
		//position holders for genome indexing, incremented as walk the length of each genome
		int g1 = 0;
		int g2 = 0;
		
		//innovation number holder, used for comparison in loop.
		int iID1 = 0;
		int iID2 = 0;
		
		//length of the longest genome
		int longest = 0;
		
		//multipliers used to tweek final speciation score
		double mDisjoint = ((ParameterSet_Evolution_NEAT)(this.getOwner().getOwner().getTemplate())).getDisjointModifier();
		double mExcess = ((ParameterSet_Evolution_NEAT)(this.getOwner().getOwner().getTemplate())).getExcessModifier();
		double mMatched = ((ParameterSet_Evolution_NEAT)(this.getOwner().getOwner().getTemplate())).getMatchedModifier();
		
		
		while ( (g1 < genomeLinks.size()-1) || (g2 < genome.genomeLinks.size()-1) )
		{
			//End of genome one reached, but not genome two so increment excess score
			if(g1 == genomeLinks.size()-1)
			{
				++g2;
				++numExcess;
				continue;
			}
			
			//End of genome two reached, but not genome one so increment excess score
			if(g2 == genome.genomeLinks.size()-1)
			{
				++g1;
				++numExcess;
				continue;
			}
			
			iID1 = genomeLinks.elementAt(g1).innovationID;
			iID2 = genome.genomeLinks.elementAt(g2).innovationID;
			
			//innovation numbers match, increase matched score
			if(iID1 == iID2)
			{	
				//calculate weight difference between these two genes
				weightDifference += Math.abs(genomeLinks.elementAt(g1).weight - genome.genomeLinks.elementAt(g2).weight);
				
				//increase matched score
				++numMatched;
				
				//move on to next gene in genome
				++g1;
				++g2;
			}
			
			//innovation numbers are different so increment the disjoint score
			if(iID1 < iID2)
			{
				++numDisjoint;
				++g1;
			}
			if(iID1 > iID2)
			{
				++numDisjoint;
				++g2;
			}
		}//end while loop
		
		//get the longest genome length
		longest = genome.genomeLinks.size();
		if(genomeLinks.size() > longest)
		{
			longest = genomeLinks.size();
		}
		
		//finally calculate the scores
		return (((mExcess * numExcess)/(double)longest) + ((mDisjoint * numDisjoint)/(double)longest) + ((mMatched * weightDifference)/numMatched));
	}
	
	public RecurrentNeuralNetwork createPhenotype()
	{
		//This is the Recurrent NN structure, the activation function is for the weights on all the arcs
		//from the bias node to each node in the network.
		RecurrentNeuralNetwork brain = new RecurrentNeuralNetwork( /*new ActivationFunction_logisticSigmoid()*/);
		//Contains a list of bias values for the Neural Network object
		ArrayList<Double> biasvalues = new ArrayList<Double>();
		//This is the vector of created neurons for the phenotype, this is requried to register all the arcs.
		Vector<Neuron> neurons = new Vector<Neuron>();
		//This hashmap contains one entry for every unique Innovation ID in the genomes Node List that are
		//enabled. The Innovation ID acts as the key value for a map to the appropriate neuron in the phenotypes
		//neuron list. This facilities linear time registration of all the arcs in the neural network phenotype
		HashMap<Integer,Integer> iIDToANNNeuron = new HashMap<Integer,Integer>();
		double ar; //activation response for the bias neurons
		int inputs = 1; //number of inputs in the phenotype
		int outputs = 1; //number of outputs in the phenotype
		int hidden = 1; //number of hidden nodes in the phenotype
		boolean biasSet = false;
		
		//Sort the Neurons and Links, they both sort on Innovation ID
		this.sort();
		
		for(int i = 0; i < genomeNeurons.size(); i++)
		{
			//Report an error while attempting to insert a node of type None
			if(genomeNeurons.get(i).getNeuronType() == 0)
			{
				//genomeNeurons.get(i).displayInformation();
				Display.error(""+this.getClass().getName()+"::createPhenotype - Node at " + i + " in genotype is invalid");
			}
			//Create neurons for input nodes
			if(genomeNeurons.get(i).getNeuronType() == 1 && genomeNeurons.get(i).isEnabled())
			{
				RecurrentNeuron neuron1 = new RecurrentNeuron( brain, new ActivationFunction_logisticSigmoid(genomeNeurons.get(i).getActivationResponse()), "in"+inputs);
				neuron1.setNGene((Element_NEAT_NGene)genomeNeurons.get(i).clone());
				neurons.add(neuron1);
				brain.registerInputNeuron(neurons.elementAt(neurons.size()-1));	
				iIDToANNNeuron.put(genomeNeurons.elementAt(i).neuronID, neurons.size()-1);
				inputs++;
			}
			//Create neurons for output nodes
			if(genomeNeurons.get(i).getNeuronType() == 2 && genomeNeurons.get(i).isEnabled())
			{
				RecurrentNeuron neuron2 = new RecurrentNeuron( brain, new ActivationFunction_logisticSigmoid(genomeNeurons.get(i).getActivationResponse()), "out"+outputs);
				neuron2.setNGene((Element_NEAT_NGene)genomeNeurons.get(i).clone());
				neurons.add(neuron2);
				brain.registerOutputNeuron(neurons.elementAt(neurons.size()-1));
				iIDToANNNeuron.put(genomeNeurons.elementAt(i).neuronID, neurons.size()-1);
				outputs++;
			}
			//Create neurons for hidden nodes
			if(genomeNeurons.get(i).getNeuronType() == 3 && genomeNeurons.get(i).isEnabled())
			{
				RecurrentNeuron neuron3 = new RecurrentNeuron( brain, new ActivationFunction_logisticSigmoid(genomeNeurons.get(i).getActivationResponse()), "hidden"+hidden);
				neuron3.setNGene((Element_NEAT_NGene)genomeNeurons.get(i).clone());
				neurons.add(neuron3);
				iIDToANNNeuron.put(genomeNeurons.elementAt(i).neuronID, neurons.size()-1);
				hidden++;
			}
			//Create an array of bias values based on the bias node in the genome
			if(genomeNeurons.get(i).getNeuronType() == 4 && genomeNeurons.get(i).isEnabled())
			{
				//WARNING Not too sure if this is right...
				ar = genomeNeurons.get(i).getActivationResponse();
				for(int k = 0; k < genomeNeurons.size(); k++)
					biasvalues.add(ar);
				biasSet = true;
			}
		}
		
		//Now register all the Arcs from the genomeLink vector.
		for(int i = 0; i < genomeLinks.size(); i++)
		{
			if(genomeLinks.elementAt(i).isEnabled())
			{
				WeightedArc arc = new WeightedArc(neurons.elementAt(iIDToANNNeuron.get(genomeLinks.elementAt(i).getFromNeuron())),
												  neurons.elementAt(iIDToANNNeuron.get(genomeLinks.elementAt(i).getToNeuron())),
												  genomeLinks.elementAt(i).getLinkWeight());
				arc.setLGene((Element_NEAT_LGene)genomeLinks.get(i).clone());
				brain.registerArc(arc);
			}
		}
		
		

		//initialize the neural network.
		brain.initNetwork();
		
		//set the bias values. Not required as the bias is hard coded to 1. Keeping the code here for future modifications.
//		if(biasSet)
//		{
//			brain.setBiasArcsWeightValues(biasvalues);
//		}
//		else
//		{
//			for(int k = 0; k < genomeNeurons.size(); k++)
//				biasvalues.add(0.0);
//		}
		
		//Some quick cleanup trying to force garbage collection here
		neurons = null;
		biasvalues = null;
		iIDToANNNeuron = null;
		//return the completed phenotype
		return brain;
		
	}
	
	/**Specifically to override the performVariations() function from Individual
	 * NEAT requires this due to its non standard genome*/
	public void performVariations()
	{
		for(int i = 0; i < getElementListSize(); i++)
			getElementAt(i).performVariations();
		for ( int i = 0 ; i != this._variationOperatorList.size() ; i++ )
			((VariationOperator)this._variationOperatorList.get(i)).applyOperator(this);
		this._variationOperatorList.clear();
	}
	
	/**Specifically to override the setFitness() function from Individual*/
	public void setFitness( double d )
	{
		setAdjustedFitness(d);
	}
	
	/**Specifically to override the updateFitness() function from Individual*/
	public void updateFitness( double value )
	{
		this.genomeRawFitness += value;
	}
	
	/**Specifically to override the getElementAt() function from Individual
	If the nodes and links were stored in an individual vector we order <Nodes> + <Links>*/
	public Element getElementAt( int index )
	{
		if(index < genomeNeurons.size())
			return genomeNeurons.elementAt(index);
		else if(index >= genomeNeurons.size() && index < getElementListSize())
			return genomeLinks.elementAt(index-genomeNeurons.size());
		else
		{
			Display.error(""+this.getClass().getName()+"::getElementAt - element at " + index + " does not exist");
			return null;
		}
	}
	
	/**Specifically to override the getElementListSize() function from Individual*/
	public int getElementListSize ()
	{
		return genomeNeurons.size()+genomeLinks.size();
	}
	
	/**It is required to override this by definition but the NEAT implementation will not use this specific function*/
	public void addElement(Element __element) {
		//Do Nothing
		Display.error(""+this.getClass().getName()+"::addElement - NEATIndividual this should NOT be called");
	}
	
	public Object clone ()
	{
		Individual_NEATIndividual individualClone = null;
		
		try 
		{
			individualClone = (Individual_NEATIndividual)super.clone();
		}
		catch (InternalError e) 
		{
            // This should never happen
            throw new InternalError(e.toString());
		}
		
		//1. Clone the unique ID
		individualClone.genomeID = genomeID;
		//2. Clone the entire Genome
		individualClone.genomeNeurons = new Vector<Element_NEAT_NGene>();
		for(int i = 0; i < genomeNeurons.size(); i++)
			individualClone.genomeNeurons.add((Element_NEAT_NGene)this.getNeuronAt(i).clone());
		individualClone.genomeLinks = new Vector<Element_NEAT_LGene>();
		for(int i = 0; i < genomeLinks.size(); i++)
			individualClone.genomeLinks.add((Element_NEAT_LGene)(this.genomeLinks.get(i)).clone());
		individualClone.genomeNetDepth = genomeNetDepth;
		individualClone.genomeRawFitness = genomeRawFitness;
		individualClone.genomeAdjustedFitness = genomeAdjustedFitness;
		individualClone.amountToSpawn = amountToSpawn;
		individualClone.inputs = inputs;
		individualClone.outputs = outputs;
		individualClone.genomeSpecies = genomeSpecies;

		return (individualClone);
	}
	
	public void sort()
	{
		try
		{
			Collections.sort(genomeNeurons);
			Collections.sort(genomeLinks);
			rebuildIIDLookup();
		}
		catch(ClassCastException e)
		{
			Display.error(""+this.getClass().getName()+"::sort - Something went wrong while sorting the population");
		}
	}
	
	//this sort should result in the list being from greatest to least
	//it is contrary to natural ordering in java from least to greatest.
	public int compareTo(Individual_NEATIndividual _o) {
		if(this.genomeRawFitness < _o.genomeRawFitness)
		{
			return -1;
		}
		else if(this.genomeRawFitness > _o.genomeRawFitness)
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
}
