/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.representation;

import java.util.*;

import simbad.picoevo.core.representation.*;
import simbad.picoevo.ext.evolution.*;
import simbad.picoevo.toolbox.*;

public class Population_NEATPopulation extends Population {

	public EvolveOperator_Population_NEATPopulation evolveOperator;
	public Population_NEATPopulation_InnovationList innovationList;
	public Population_NEATPopulation_SpeciesList speciesList;
	public int populationSize; //the size of the population, equal to the initial size
	public int nextGenomeID; //the next valid unique genome ID number
	public double totalFitnessAdjusted;
	public double averageFitnessAdjusted;
	public double bestFitnessEver;
	
	public Population_NEATPopulation(String name, World space) {
		super(name, space);
		evolveOperator = new EvolveOperator_Population_NEATPopulation();
		innovationList = new Population_NEATPopulation_InnovationList();
		speciesList = new Population_NEATPopulation_SpeciesList(this);
		populationSize = this.getOwner().getTemplate().getInitPopSize();
		nextGenomeID = this.getOwner().getTemplate().getInitPopSize(); //should always contain the next newest unique genome ID.
		totalFitnessAdjusted = 0;
		averageFitnessAdjusted = 0;
		bestFitnessEver = 0;
	}

	public void displayInformation()
	{
		Display.info(" ############################################################################################# ");
		Display.info(" ##### Displaying Full Population Information : " + this.getClass() + " #");
		Display.info(" ############################################################################################# ");
		super.displayInformation();
		speciesList.displayInformation();
		innovationList.displayInformation();
		Display.info(" ############################################################################################# ");
	}
	
	public void displayStatisticsNoGUI() 
	{
		((StatisticsOperator_NEAT)this.getTemplate().getPopulationStatisticsOperator()).displayStatisticsNoGUI(this);
	}
	
	public void evolveOneStep(boolean display)
	{
		this.evolveOperator.evolve(this, display);
	}

	public void logGeneration(String generationString)
	{
		if ( (ParameterSet_Evolution_NEAT)this.getTemplate() != null )
			((StatisticsOperator_NEAT)(this.getTemplate().getPopulationStatisticsOperator())).logGeneration(this,generationString);
	}
	
	public void logExperiment()
	{
		if ( (ParameterSet_Evolution_NEAT)this.getTemplate() != null )
			((StatisticsOperator_NEAT)(this.getTemplate().getPopulationStatisticsOperator())).logExperiment(this);
	}
	
	public void logXORExperiment(String generationString)
	{
		if ( (ParameterSet_Evolution_NEAT)this.getTemplate() != null )
			((StatisticsOperator_NEAT)(this.getTemplate().getPopulationStatisticsOperator())).logXORExperiment(this,generationString);
	}
	
	public Individual_NEATIndividual getRawBest()
	{
		double highest = 0.0;
		int highestLocation = 0;

		for(int i = 0; i < this._individualList.size(); i++)
		{
			if(((Individual_NEATIndividual)this._individualList.get(i)).getRawFitness() > highest)
			{
				highest = ((Individual_NEATIndividual)this._individualList.get(i)).getRawFitness();
				highestLocation = i;
			}
		}
		return ((Individual_NEATIndividual)this._individualList.get(highestLocation));
	}
	
	public Individual_NEATIndividual getAdjustedBest()
	{
		double highest = 0.0;
		int highestLocation = 0;
		
		for(int i = 0; i < this._individualList.size(); i++)
		{
			if(((Individual_NEATIndividual)this._individualList.get(i)).getAdjustedFitness() > highest)
			{
				highest = ((Individual_NEATIndividual)this._individualList.get(i)).getAdjustedFitness();
				highestLocation = i;
			}
		}
		return ((Individual_NEATIndividual)this._individualList.get(highestLocation));
	}
	
	public void sort()
	{
		try
		{
			Collections.sort(super._individualList);
		}
		catch(ClassCastException e)
		{
			Display.error(""+this.getClass().getName()+"::sort - Something went wrong while sorting the population");
		}
	}
}
