/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Updated by: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */


/*
 * Author: nicolas.bredeche(@)lri.fr
 * Created on 16 fvr. 2006
 * 
 */

package simbad.piconode.core.node;

import java.util.*;

import simbad.piconode.XmlRepresentation;
import simbad.piconode.core.arc.*;
import simbad.piconode.core.operator.*;
import simbad.picoevo.ext.representation.*;
import simbad.picoevo.toolbox.Display;
import simbad.piconode.ext.*;

/**
 * The Network class provides the minimal basis to define a network module. 
 */
public abstract class Network extends Module implements XmlRepresentation, java.io.Serializable{

    /*
     * Data
     */

    protected ArrayList _arcList = new ArrayList(); // std as in "standard". Means that it contains no arcs from bias neuron
	protected ArrayList _nodeList = new ArrayList();
    
    
    /*
     * Constructors
     */


    /*
     * Methods
     */

	protected ConstrainedArc getArcAt( int __index )
	{
	    if ( __index >= getArcListSize() || __index < 0 )
	    {
	        System.out.println("Network::getArcAt(-) - out of range");
	        System.exit(-1);
	    }
	    return((ConstrainedArc)_arcList.get(__index));
	}
	
	protected int getNodeListSize ()
	{
	    return ( _nodeList.size() );
	}
	
	protected ConnectedNode getNodeAt (int __index)
	{
	    return ( (ConnectedNode)_nodeList.get(__index) );
	}

	/**
	 * return the number of Arcs (if NeuralNetwork : excepting arcs from bias neuron (may be useful when loading/saving weights (e.g. for evolution)))
	 * @return
	 */
	public int getArcListSize ()
	{
	    return (_arcList.size());
	}

	/**
     * register an arc into the network (if NeuralNetwork : arcs from the bias neuron are automaticaly registered)
     * @param __arc
     */
    public void registerArc( ConstrainedArc __arc )
	{
	    _arcList.add(__arc);
	}

    
    /**
	 * display information on the network
	 *
	 */
	public void displayInformation()
	{
	    System.out.println("### Network Information : summary ###");
	    for ( int i = 0 ; i != this.getNodeListSize() ; i++ )
	        (this.getNodeAt(i)).displayInformation();
	}

    protected void registerNode(ConnectedNode __neuron)
    {
        _nodeList.add(__neuron);
    }
    
    //This currently ONLY works with ActivationFunction_logisticSigmoid
    //otherwise the learned activation response k values will be dumped
    public void updateGenotype( Individual_NEATIndividual untrained)
    {	
    	//check for bias node in original genotype
    	//save it and its location
    	Element_NEAT_NGene bias = null;
    	int biasLocation = 0;
    	boolean biasExists = false;
    	
    	for( int i = 0; i < untrained.getNumNeurons(); i++)
    	{
    		if(untrained.getNeuronAt(i).getNeuronType() == 4 /* bias type */ )
    		{
    			biasExists = true;
    			biasLocation = i;
    			bias = untrained.getNeuronAt(i);
    		}
    	}
    	
    	//get all nodes genes from the phenotype nodes
    	//set the new activation responses as you get the nodes
    	
    	Vector<Element_NEAT_NGene> newNodes = new Vector<Element_NEAT_NGene>();
    	
    	for(int node = 0; node < this.getNodeListSize(); node++)
    	{
    		if(this.getNodeAt(node).getName() != "bias")
    		{
    		newNodes.add(this.getNodeAt(node).getNGene());
    		if(((Neuron)this.getNodeAt(node)).getActivationFunction() instanceof ActivationFunction_logisticSigmoid)
    			newNodes.elementAt(newNodes.size()-1).setActivationResponse(((ActivationFunction_logisticSigmoid)(((Neuron)this.getNodeAt(node)).getActivationFunction())).getSigmoidKValue());
    		}
    	}
    	
    	//replace the bias node if it exists
    	//the biasLocation check is due to the way NEAT handles its bias node at the end of its node vector
    	if( biasExists && biasLocation == newNodes.size())
    	{
    		newNodes.add(bias);
    	}
    	
    	//get all the link genes from the phenotype links
    	//set the new weights as you get the links
    	
    	Vector<Element_NEAT_LGene> newLinks = new Vector<Element_NEAT_LGene>();
    	
    	for(int link = 0; link < this.getArcListSize(); link++)
    	{
    		newLinks.add(this.getArcAt(link).getLGene());
    		newLinks.elementAt(newLinks.size()-1).setLinkWeight(((WeightedArc)this.getArcAt(link)).getWeightValue());
    	}
    	//update the individuals links and nodes
    	untrained.setAllLinks(newLinks);
    	untrained.setAllNeurons(newNodes);
    	
    	//a little cleanup
    	newLinks = null;
    	newNodes = null;
    	bias = null;
    }

    /**
     * Thomas Darde TER NNEDITOR
     * Provides an xml representation of this element
     * Xml DTD is defined in visual piconode
     */
    
	public String toXml(){
		String retour="<file>\n<nodes>\n";
	if(_nodeList.size()==0)
		System.out.println("node list is empty");
	   for ( int i = 0 ; i < _nodeList.size() ; i++ )
	        retour+=((Neuron) _nodeList.get(i)).toXml()+"\n";
	   retour+="</nodes>\n<links>\n";
	   for ( int i = 0 ; i < _arcList.size() ; i++ )
	        retour+=((WeightedArc) _arcList.get(i)).toXml()+"\n";
	   retour+="</links>\n</file>\n";
	   return retour;
	}




}
