/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Updated by: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */

/*
 * Author: nicolas.bredeche(@)lri.fr
 * Created on 16 fvr. 2006
 * 
 */

package simbad.piconode.core.node;

import java.util.ArrayList;

import simbad.piconode.XmlRepresentation;
import simbad.picoevo.ext.representation.*;

/** 
 * General Node class from which is derived every elements from atomic node 
 * (e.g. neurons, bayesian node) to modules (e.g. neural nets, bayesian nets, 
 * Dynamic Systems implementations, etc.)
 */
public abstract class Node implements XmlRepresentation, java.io.Serializable{

	/* ### data ### */
	
    protected	String	_name = new String("(node)");
    protected   Element_NEAT_NGene nGene;

    /* ### Constructor(s) ### */
    
    public Node ()
    {
    }
    
    /* ### methods ### */
    
    final public void setName (String __name)
    {
        this._name = __name;
    }

    final public String getName ()
    {
        return ( this._name );
    }
    
	public Element_NEAT_NGene getNGene() {
		return nGene;
	}

	public void setNGene(Element_NEAT_NGene gene) {
		nGene = gene;
	}

	/**
	* display information 
	*/
	abstract public void displayInformation();
	
	/**
	 * compute the output of a node/module
	 */
	abstract public void step ();
	
	/**
	 * Thomas Darde TER NNEditor
	 * Allows an object to give an XML representation from himself
	 */
	 /**
     * Thomas Darde TER NNEDITOR
     * Provides an xml representation of this element
     * Xml DTD is defined in visual piconode
     */
    public String toXml(){
		String xmlString;
		xmlString="<node>";
		xmlString+="<name at=\"" + this.getName() +"\"/>"
			+ "<id at=\"" + this.hashCode() +"\"/>";
		xmlString+= "</node>";
		return xmlString;
    }
}
