/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Updated by: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */

/*
 * Created on 6 janv. 2005
 * bredeche(at)lri(point)fr
 * 
 */

package simbad.piconode.ext;

import simbad.piconode.core.*;
import simbad.piconode.core.operator.ActivationFunctionWithDerivative;

public class ActivationFunction_logisticSigmoid extends ActivationFunctionWithDerivative implements java.io.Serializable{

    /*
     * Data
     */
    

    protected	static	double _sigmoidDefaultKValue = 1; // default parameter value 
    private	double	_sigmoidKValue; // more or less flat sigmoid (->0) ou "threshlold" (parameter "k")
    
    /*
     * Constructor(s)
     */

    public ActivationFunction_logisticSigmoid () 
    {
        _sigmoidKValue = getDefaultSigmoidKValue(); 
    }

    public ActivationFunction_logisticSigmoid ( double __value ) 
    {
        _sigmoidKValue = __value; 
    }

    /*
     * Method(s)
     */
    
    public double getSigmoidKValue ()
    {
    	return ( _sigmoidKValue );
    }
    
	static protected double getDefaultSigmoidKValue ()
	{
	    return ( _sigmoidDefaultKValue );
	}

    public double applyFunction( double __value ) 	// f(x) = 1 / ( 1 + e^(-k*x)) [ sigmoid function - f(x) is [0,1] ]
	{
   	    return( 1 / ( 1 + Math.exp( - _sigmoidKValue * __value )) );
	}
    
    public double applyDerivativeFunction( double __value ) // f'(x) = x * ( 1 - x )
    {
    	return( __value * ( 1 - __value ) );
    }

}