/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.evolution;

import simbad.picoevo.core.evolution.*;
import simbad.picoevo.core.representation.*;
import simbad.picoevo.ext.representation.*;

public class VariationOperator_Element_NEAT_Gene extends VariationOperator implements Cloneable{

	protected double probabilityRateReplaced;
	protected double maxWeightPerturbation;
	protected double activationMutationRate;
	protected double maxActivationPerturbation;
	
	public VariationOperator_Element_NEAT_Gene(String __name) {
		super(__name);
		probabilityRateReplaced = 0.0;
		maxWeightPerturbation = 0.0;
		activationMutationRate = 0.0;
		maxActivationPerturbation = 0.0;
	}

	public VariationOperator_Element_NEAT_Gene(String __name, double rate, double rateReplaced, double maxPert,
												double actRate, double maxAct) {
		super(__name, rate);
		probabilityRateReplaced = rateReplaced;
		maxWeightPerturbation = maxPert;
		activationMutationRate = actRate;
		maxActivationPerturbation = maxAct;
	}

	public void applyOperator(EvolutionObject object) 
	{
		if(object instanceof Element_NEAT_LGene)
		{
			if(Math.random() < _operatorRate)
			{
				if(Math.random() < probabilityRateReplaced)
					((Element_NEAT_LGene)object).setLinkWeight(Math.random()-Math.random()); //sets value between -1.0 and 1.0
				else
					((Element_NEAT_LGene)object).setLinkWeight(((Element_NEAT_LGene)object).getLinkWeight()+
																((Math.random()-Math.random())*maxWeightPerturbation));
			}
		}
		if(object instanceof Element_NEAT_NGene)
		{
			if(Math.random() < activationMutationRate)
			{
				((Element_NEAT_NGene)object).setActivationResponse(((Element_NEAT_NGene)object).getActivationResponse()+
																	((Math.random()-Math.random())*maxActivationPerturbation));
			}
		}

	}
	
	public Object clone() {
		
		VariationOperator_Element_NEAT_Gene op = null;
		
		try
		{
			op = (VariationOperator_Element_NEAT_Gene)super.clone();
		}
		catch( InternalError e)
		{
			// This should never happen
            throw new InternalError(e.toString());
		}
		op._name = super.getName();
		op.setRate(super._operatorRate);
		op.probabilityRateReplaced = probabilityRateReplaced;
		op.maxWeightPerturbation = maxWeightPerturbation;
		op.activationMutationRate = activationMutationRate;
		op.maxActivationPerturbation = maxActivationPerturbation;
		return op;
	}
}
