/*
 *  NS - Neuroevolutionary Solver
 *  Copyright (C) 2008 Robert Lucas
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 -----------------------------------------------------------------------------
 * $Author: Robert Lucas $ 
 * $Date: 2008/10/11 17:20:00 $
 * $Revision: 1.00 $
 */
package simbad.picoevo.ext.representation;

import java.util.ArrayList;

import simbad.picoevo.core.evolution.VariationOperator;
import simbad.picoevo.core.representation.Element;
import simbad.picoevo.core.representation.Individual;
import simbad.picoevo.toolbox.Display;

public abstract class Element_NEAT_Gene extends Element implements Comparable, Cloneable, java.io.Serializable{

	protected int innovationID;
	protected boolean recurrent;
	protected boolean enabled;
	
	public Element_NEAT_Gene() {
		super();
		innovationID = -1;
		recurrent = false;
		enabled = true;
	}
	
	public Element_NEAT_Gene(String __name, Individual owner, VariationOperator levelOperator) {
		super(__name, owner, levelOperator);
		innovationID = -1;
		recurrent = false;
		enabled = true;
	}

	public Element_NEAT_Gene(String __name, Individual owner, VariationOperator levelOperator,
								int _innovationID, boolean _recurrent) 
	{
		super(__name, owner, levelOperator);
		innovationID = _innovationID;
		recurrent = _recurrent;
		enabled = true;
	}

	public Element_NEAT_Gene(String __name, Individual owner, ArrayList levelOperatorList,
								int _innovationID, boolean _recurrent) 
	{
		super(__name, owner, levelOperatorList);
		innovationID = _innovationID;
		recurrent = _recurrent;
		enabled = true;
	}
	
	public Element_NEAT_Gene(Individual owner, VariationOperator levelOperator) {
		super(owner, levelOperator);
		innovationID = -1;
		recurrent = false;
		enabled = true;
	}

	public Element_NEAT_Gene(Individual owner, ArrayList levelOperatorList) {
		super(owner, levelOperatorList);
		innovationID = -1;
		recurrent = false;
		enabled = true;
	}
	
	public Element_NEAT_Gene(Individual owner, VariationOperator levelOperator,
								int _innovationID, boolean _recurrent) 
	{
		super(owner, levelOperator);
		innovationID = _innovationID;
		recurrent = _recurrent;
		enabled = true;
	}

	public Element_NEAT_Gene(Individual owner, ArrayList levelOperatorList,
								int _innovationID, boolean _recurrent) 
	{
		super(owner, levelOperatorList);
		innovationID = _innovationID;
		recurrent = _recurrent;
		enabled = true;
	}

	public void setInnovationID(int _i) {
		this.innovationID = _i; 
	}
	
	public int getInnovationID() {
		return this.innovationID; 
	}
	
	public void setRecurrent(boolean _b) {
		this.recurrent = _b;
	}
	
	public boolean getRecurrent() {
		return this.recurrent;
	}
	
	public void enable() {
		this.enabled = true;
	}
	
	public void disable() {
		this.enabled = false;
	}
	
	public boolean isEnabled() {
		return this.enabled;
	}
	
	public void performVariations ()
	{
		super.performVariations();
		this._variationOperatorList.clear();	
	}
	
	public void displayInformation()
	{
		//This super call will display the pending operations on this genome, not exactly required
		//super.displayInformation();
		Display.info_nocr(" ( \""+_name+"\",{");
		Display.info_nocr(" innovationID: "+innovationID);
		Display.info_nocr(" recurrent: "+recurrent);
		Display.info_nocr(" enabled: "+enabled);
		Display.info_nocr(" )");
	}
	
	public String toString()
	{
		StringBuffer output = new StringBuffer();
		output.append(_name);
		output.append(" / iID: ");
		output.append(innovationID);
		output.append(" - R:");
		output.append(recurrent);
		output.append(" E:");
		output.append(enabled);
		return output.toString();
	}
	
	public Object clone()
	{
		Element_NEAT_Gene g = null;
		
		try 
		{
			g = (Element_NEAT_Gene)super.clone();
		}
		catch (InternalError e) 
		{
            // This should never happen
            throw new InternalError(e.toString());
		}
		g._individualOwner = super.getOwner();
		g.innovationID = innovationID;
		g.recurrent = recurrent;
		g.enabled = enabled;
		
		return g;
	}
	
	public int compareTo(Element_NEAT_Gene _o) {
		return this.innovationID - _o.innovationID;
	}
}
